/***************************************************************************
                          oscevent.cpp  -  description
                             -------------------
    begin                : Tue Feb 11 2003
    copyright            : (C) 2003 by Ivica Bukvic
    email                : ico@fuse.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* OSC License
Copyright (c) 1996,1997.  The Regents of the University of California (Regents).
All Rights Reserved.

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes, without
fee and without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following two paragraphs
appear in all copies, modifications, and distributions.  Contact The Office of
Technology Licensing, UC Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley,
CA 94720-1620, (510) 643-7201, for commercial licensing opportunities.

Written by Matt Wright, The Center for New Music and Audio Technologies,
University of California, Berkeley.

     IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
     SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
     ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
     REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

     REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
     LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
     DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
     REGENTS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
     ENHANCEMENTS, OR MODIFICATIONS.
*/
 
#include "oscevent.h"

#include <qmessagebox.h>

OSCEvent::OSCEvent(QWidget* parent, const char* WidgetName, WFlags fl, const char *host,
        int port, bool typetags, char *msgname, int low_a, int high_a, int low_b, int high_b, int low_c, int high_c )
{
    connect ( this, SIGNAL ( Error(QString) ), parent, SLOT ( Error(QString) ) );
    tmpBuf = new char[SC_BUFFER_SIZE];
    OSC_initBuffer(buf, SC_BUFFER_SIZE, tmpBuf);
    hostname = const_cast<char *>(host);
    //qDebug("HOSTNAME:");
    //qDebug( hostname );
    setName ( WidgetName );
    portname = port;
    status = -1;
    tags = typetags;
    qDebug ( name() );
    //qDebug ( "port = %d", portname );

    if ( (QString) msgname == "null" )
    {
        //qDebug("Creating oscroute");
        la = low_a;
        ha = high_a;
        lb = low_b;
        hb = high_b;
        lc = low_c;
        hc = high_c;
        OpenSocket();
    }
    else
    {
        //qDebug("Creating instance");
        la = -1;
        ha = -1;
        lb = -1;
        hb = -1;
        lc = -1;
        hc = -1;
        OpenSocket();
        //qDebug("Instant: status = %d", status );
        //qDebug("params %d %d", low_a, high_a );
        if ( status )
        {
            if ( low_a != -100 )
            {
                WriteMessage( msgname, low_a, high_a );
            }
            else
            {
                //qDebug("osccustom");
                WriteCustomMessage( msgname );
            }
        }
        //qDebug("Done");
    }
}

OSCEvent::~OSCEvent()
{
    //if (status)
    //{
      //close(sock);
      //qDebug("Closing socket");
    //}
    //qDebug("Deleting");
}

void OSCEvent::OpenSocket()
{
  if ( portname > 0 )
  {
      //qDebug("calling openudpsocket");
      status = OpenUDPSocket();
      //qDebug("result of the call %d", status );
  }
  else
  {
    status = -1;
  }
  //qDebug( "status = %d", status );
}

int OSCEvent::GetStatus()
{
  return status;
}

void OSCEvent::OpenBundle()
{
  if (OSC_openBundle(buf, OSCTT_Immediately())) {
      SendError("Problem opening bundle");
      //status = 0;
  }
  /*
  else
  {
    status = 1;
  }
  */
}

void OSCEvent::CloseBundle()
{
  if (status)
  {
    if (OSC_closeBundle(buf))
    {
      SendError("Problem closing bundle");
      //status = 0;
    }
  }
}

int OSCEvent::ParseMessage(OSCbuf *buf, char *messageName, int numArgs, typedArg *args)
{
    int j, returnVal;
    returnVal = 0;

    char typeTags[MAX_ARGS+2];
    int i;

    typeTags[0] = ',';

    for (i = 0; i < numArgs; ++i) {
        switch (args[i].type) {
            case INT:
            typeTags[i+1] = 'i';
            break;

            case FLOAT:
            typeTags[i+1] = 'f';
            break;

            case STRING:
            typeTags[i+1] = 's';
            break;
        }
    }
    typeTags[i+1] = '\0';

    returnVal = OSC_writeAddressAndTypes(buf, messageName, typeTags);
    if (returnVal) {
        SendError("Problem writing address");
        return -1;
    }

    for (j = 0; j < numArgs; j++) {
        switch (args[j].type) {
            case INT:
            if ((returnVal = OSC_writeIntArg(buf, args[j].datum.i)) != 0) {
                return returnVal;
            }
            break;

            case FLOAT:
            if ((returnVal = OSC_writeFloatArg(buf, args[j].datum.f)) != 0) {
                return returnVal;
            }
            break;

            case STRING:
            if ((returnVal = OSC_writeStringArg(buf, args[j].datum.s)) != 0) {
                return returnVal;
            }
            break;
        }
    }
    return returnVal;
}

int OSCEvent::WriteMessage( char *message, int num_1, int num_2 )
{
  int returnVal;
  returnVal = 0;
  typedArg arg[2];
  if (status)
  {
    if ( message != NULL )
    {
      OpenBundle();
      if (!tags)
      {
          returnVal = OSC_writeAddress(buf, message);
          if (returnVal)
          {
              SendError("Problem writing address");
              return -1;
          }
         OSC_writeIntArg( buf, num_1 );
         if ( num_2 >= 0 ) OSC_writeIntArg( buf, num_2 );         
      }
      else
      {
          arg[0].type = INT;
          arg[0].datum.i = num_1;
          if ( num_2 == -1 )
          {
              ParseMessage(buf, message, 1, arg);
          }
          else
          {
              arg[1].type = INT;
              arg[1].datum.i = num_2;
              ParseMessage(buf, message, 2, arg );
          }
      }
      CloseBundle();
      if (!SendBuf(OSC_getPacket(buf)))
      {
         SendError("Couldn't send out socket");
         //close(sock);
         return -1;
      }
      OSC_resetBuffer(buf);
    }
    else
    {
      SendError("Nothing to write");
    }
  }
  return 0;
}

int OSCEvent::WriteCustomMessage( char *message )
{
    int returnVal, count, subcount, position, subposition, i, j, numArgs;
    QString values, wholeMsg, singleMsg;
    char *singleCommand;
    bool ok;
    typedArg arg[MAX_ARGS];

    int intArg;
    float floatArg;

    returnVal = 0;
    count = 0;
    position = 0;

    if (status)
    {
        if ( message != NULL )
        {
            wholeMsg = message;
            OpenBundle();
            //Single pipe | delineates separate messages in the bundle
            //qDebug("bundle open");
            while ( position != -1 )
            {
                //check if this is a bundle with multiple messages
                position = wholeMsg.find( "|", position+1 );
                //count will be 0 if there is only one event
                if ( position != -1 )
                {
                    count++;
                }
            }
            //qDebug("no of msgs %d", count+1 );

            position = 0;
            for ( i = 0; i <= count; i++ )
            {
                singleMsg = wholeMsg.section("|", i, i ).simplifyWhiteSpace();
                //qDebug( singleMsg );
                subposition = 0;
                subcount = 0;
                numArgs = 0;
                while ( subposition != -1 )
                {
                    subposition = singleMsg.find( " ", subposition+1 );
                    //count will be 0 if there is no args
                    if ( subposition != -1 )
                    {
                        subcount++;
                    }
                }
                //qDebug("no of args %d", subcount );
                numArgs = subcount;

                values = singleMsg.section(" ",0,0 );
                //qDebug( "values = " + values );
                singleCommand = QStringToChar( values );

                //qDebug( singleCommand );

                if (!tags)
                {
                    returnVal = OSC_writeAddress(buf, singleCommand);
                    if (returnVal)
                    {
                        SendError("Problem writing address");
                        return -1;
                    }
                }
                if ( subcount > 0 )
                {
                    //we got some args
                    for ( j = 0; j < subcount; j++ )
                    {
                        values = singleMsg.section(" ", j+1, j+1);
                        //qDebug( "Argument: " + values );
                        intArg = values.toInt( &ok );
                        if ( !ok )
                        {
                            floatArg = values.toFloat( &ok );
                            if ( !ok )
                            {
                                arg[j].type = STRING;
                                arg[j].datum.s = QStringToChar( values );
                                //qDebug( arg[j].datum.s );
                                if (!tags)
                                {
                                    OSC_writeStringArg( buf, arg[j].datum.s );
                                }
                            }
                            else
                            {
                                arg[j].type = FLOAT;
                                arg[j].datum.f = floatArg;
                                if (!tags)
                                {
                                    OSC_writeFloatArg( buf, arg[j].datum.f );
                                }
                            }
                        }
                        else
                        {
                            arg[j].type = INT;
                            arg[j].datum.i = intArg;
                            if (!tags)
                            {
                                OSC_writeIntArg( buf, arg[j].datum.i );
                            }
                        }
                    }
                }
                if (tags)
                {
                    ParseMessage(buf, singleCommand, numArgs, arg);
                }
                //deallocate memory
                delete singleCommand;
                for ( j = 0; j < subcount; j++ )
                {
                    if ( arg[j].type == STRING )
                    {
                        delete arg[j].datum.s;
                    }
                }
            }
            CloseBundle();
            if (!SendBuf(OSC_getPacket(buf)))
            {
               SendError("Couldn't send out socket");
               return -1;
            }
            OSC_resetBuffer(buf);
        }
        else
        {
          SendError("Nothing to write");
        }
    }
    return 0;
}

void OSCEvent::SendError( QString text )
{
    QString msg;
    msg = "OSC real-time event no." + (QString) name() + " reporting error: " + text + "...\n";
    Error( msg );
}

char* OSCEvent::QStringToChar( QString s )
{
    return strcpy( new char[s.length()+1], s.latin1() );
}

/////////////////////////////////NETWORK///////////////////////////////////////




int OSCEvent::OpenUDPSocket()
{
	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
	{
        SendError("Error -- Failed to create socket (check your network connections including localhost network service)...");
        //qDebug("failed to create socket");
        return -1;
	}

	struct hostent *he;

	bzero(&theiraddr, sizeof(theiraddr));
	theiraddr.sin_family = AF_INET;         // host byte order
	theiraddr.sin_port = htons(portname);     // short, network byte order
	if ((he=gethostbyname(hostname)) == NULL)
	{
        SendError("Failed to gethostbyname");
        //qDebug("failed to get hostname");
        return -1;
	}
	theiraddr.sin_addr = *((struct in_addr *)he->h_addr);
    //qDebug( "Everything ok" );
	return 1;
}

int OSCEvent::SendBuf( char* buffer )
{
    int tolen = sizeof(theiraddr);
    int len = OSC_packetSize(buf);

    if (sendto(sock, buffer, len, 0, (struct sockaddr *) &theiraddr, tolen)==-1)
      return 0;
    else
      return 1;
}

///////////////////////////////////OSC/////////////////////////////////////////




void OSCEvent::OSC_initBuffer(OSCbuf *buf, int size, char *byteArray)
{
    buf->buffer = byteArray;
    buf->size = size;
    OSC_resetBuffer(buf);
}

void OSCEvent::OSC_resetBuffer(OSCbuf *buf) {
    buf->bufptr = buf->buffer;
    buf->state = EMPTY;
    buf->bundleDepth = 0;
    buf->prevCounts[0] = 0;
    buf->gettingFirstUntypedArg = 0;
    buf->typeStringPtr = 0;
}

int OSCEvent::OSC_isBufferEmpty(OSCbuf *buf) {
    return buf->bufptr == buf->buffer;
}

int OSCEvent::OSC_freeSpaceInBuffer(OSCbuf *buf) {
    return buf->size - (buf->bufptr - buf->buffer);
}

int OSCEvent::OSC_isBufferDone(OSCbuf *buf) {
    return (buf->state == DONE || buf->state == ONE_MSG_ARGS);
}

char *OSCEvent::OSC_getPacket(OSCbuf *buf)
{
    return buf->buffer;
}

int OSCEvent::OSC_packetSize(OSCbuf *buf)
{
    return (buf->bufptr - buf->buffer);
}

void OSCEvent::PatchMessageSize(OSCbuf *buf)
{
    int4byte size;
    size = buf->bufptr - ((char *) buf->thisMsgSize) - 4;
    *(buf->thisMsgSize) = htonl(size);
}

int OSCEvent::OSC_openBundle(OSCbuf *buf, OSCTimeTag tt)
{
    if (buf->state == ONE_MSG_ARGS)
    {
        SendError("Can't open a bundle in a one-message packet");
        return 3;
    }

    if (buf->state == DONE)
    {
        SendError("This packet is finished; can't open a new bundle");
        return 4;
    }

    if (++(buf->bundleDepth) >= MAX_BUNDLE_NESTING)
    {
        SendError("Bundles nested too deeply; change MAX_BUNDLE_NESTING in OSCEvent.h");
        return 2;
    }

    if (CheckTypeTag(buf, '\0')) return 9;
    if (buf->state == GET_ARGS)
    {
        PatchMessageSize(buf);
    }

    if (buf->state == EMPTY)
    {
        /* Need 16 bytes for "#bundle" and time tag */
        CheckOverflow(buf, 16);
    }
    else
    {
        /* This bundle is inside another bundle, so we need to leave
        a blank size count for the size of this current bundle. */
        CheckOverflow(buf, 20);
        *((int4byte *)buf->bufptr) = 0xaaaaaaaa;
        buf->prevCounts[buf->bundleDepth] = (int4byte *)buf->bufptr;
        buf->bufptr += 4;
    }

    buf->bufptr += OSC_padString(buf->bufptr, "#bundle");
    *((OSCTimeTag *) buf->bufptr) = tt;

    if (htonl(1) != 1) {
    /* Byte swap the 8-byte integer time tag */
    int4byte *intp = (int4byte *)buf->bufptr;
    intp[0] = htonl(intp[0]);
    intp[1] = htonl(intp[1]);
    }

    buf->bufptr += sizeof(OSCTimeTag);
    buf->state = NEED_COUNT;
    buf->gettingFirstUntypedArg = 0;
    buf->typeStringPtr = 0;
    return 0;
}


int OSCEvent::OSC_closeBundle(OSCbuf *buf)
{
    if (buf->bundleDepth == 0)
    {
        /* This handles EMPTY, ONE_MSG, ARGS, and DONE */
        SendError("Can't close bundle; no bundle is open");
        return 5;
    }

    if (CheckTypeTag(buf, '\0')) return 9;
    if (buf->state == GET_ARGS)
    {
        PatchMessageSize(buf);
    }

    if (buf->bundleDepth == 1)
    {
        /* Closing the last bundle: No bundle size to patch */
        buf->state = DONE;
    }
    else
    {
        /* Closing a sub-bundle: patch bundle size */
        int size = buf->bufptr - ((char *) buf->prevCounts[buf->bundleDepth]) - 4;
        *(buf->prevCounts[buf->bundleDepth]) = htonl(size);
        buf->state = NEED_COUNT;
    }

    --buf->bundleDepth;
    buf->gettingFirstUntypedArg = 0;
    buf->typeStringPtr = 0;
    return 0;
}


int OSCEvent::OSC_closeAllBundles(OSCbuf *buf)
{
    if (buf->bundleDepth == 0)
    {
        /* This handles EMPTY, ONE_MSG, ARGS, and DONE */
        SendError("Can't close all bundles; no bundle is open");
        return 6;
    }

    if (CheckTypeTag(buf, '\0')) return 9;
    while (buf->bundleDepth > 0)
    {
        OSC_closeBundle(buf);
    }
    buf->typeStringPtr = 0;
    return 0;
}

int OSCEvent::OSC_writeAddress(OSCbuf *buf, char *name)
{
    int4byte paddedLength;

    if (buf->state == ONE_MSG_ARGS)
    {
        SendError("This packet is not a bundle, so you can't write another address");
        return 7;
    }

    if (buf->state == DONE)
    {
        SendError("This packet is finished; can't write another address");
        return 8;
    }

    if (CheckTypeTag(buf, '\0')) return 9;
    paddedLength = OSC_effectiveStringLength(name);

    if (buf->state == EMPTY)
    {
        /* This will be a one-message packet, so no sizes to worry about */
        CheckOverflow(buf, paddedLength);
        buf->state = ONE_MSG_ARGS;
    }
    else
    {
        /* GET_ARGS or NEED_COUNT */
        CheckOverflow(buf, 4+paddedLength);
        if (buf->state == GET_ARGS)
        {
            /* Close the old message */
            PatchMessageSize(buf);
        }
        buf->thisMsgSize = (int4byte *)buf->bufptr;
        *(buf->thisMsgSize) = 0xbbbbbbbb;
        buf->bufptr += 4;
        buf->state = GET_ARGS;
    }

    /* Now write the name */
    buf->bufptr += OSC_padString(buf->bufptr, name);
    buf->typeStringPtr = 0;
    buf->gettingFirstUntypedArg = 1;
    return 0;
}

int OSCEvent::OSC_writeAddressAndTypes(OSCbuf *buf, char *name, char *types)
{
    int result;
    int4byte paddedLength;

    if (CheckTypeTag(buf, '\0')) return 9;
    result = OSC_writeAddress(buf, name);
    if (result) return result;
    paddedLength = OSC_effectiveStringLength(types);
    CheckOverflow(buf, paddedLength);
    buf->typeStringPtr = buf->bufptr + 1; /* skip comma */
    buf->bufptr += OSC_padString(buf->bufptr, types);
    buf->gettingFirstUntypedArg = 0;
    return 0;
}

int OSCEvent::CheckTypeTag(OSCbuf *buf, char expectedType)
{
    if (buf->typeStringPtr)
    {
       if (*(buf->typeStringPtr) != expectedType)
       {
           if (expectedType == '\0')
           {
               SendError("According to the type tag I expected more arguments");
           }
           else if (*(buf->typeStringPtr) == '\0')
           {
               SendError("According to the type tag I didn't expect any more arguments");
           }
           else
           {
               SendError("According to the type tag I expected an argument of a different type");
               //printf("* Expected %c, string now %s\n", expectedType, buf->typeStringPtr);
           }
           return 9;
       }
    ++(buf->typeStringPtr);
    }
    return 0;
}


int OSCEvent::OSC_writeFloatArg(OSCbuf *buf, float arg)
{
    int4byte *intp;
    //int result;

    CheckOverflow(buf, 4);
    if (CheckTypeTag(buf, 'f')) return 9;
    /* Pretend arg is a long int so we can use htonl() */
    intp = ((int4byte *) &arg);
    *((int4byte *) buf->bufptr) = htonl(*intp);
    buf->bufptr += 4;
    buf->gettingFirstUntypedArg = 0;
    return 0;
}

int OSCEvent::OSC_writeFloatArgs(OSCbuf *buf, int numFloats, float *args)
{
    int i;
    int4byte *intp;
    
    CheckOverflow(buf, 4 * numFloats);
    /* Pretend args are long ints so we can use htonl() */
    intp = ((int4byte *) args);

    for (i = 0; i < numFloats; i++)
    {
        if (CheckTypeTag(buf, 'f')) return 9;
        *((int4byte *) buf->bufptr) = htonl(intp[i]);
        buf->bufptr += 4;
    }
    buf->gettingFirstUntypedArg = 0;
    return 0;
}

int OSCEvent::OSC_writeIntArg(OSCbuf *buf, int4byte arg)
{
    CheckOverflow(buf, 4);
    if (CheckTypeTag(buf, 'i')) return 9;

    *((int4byte *) buf->bufptr) = htonl(arg);
    buf->bufptr += 4;

    buf->gettingFirstUntypedArg = 0;
    return 0;
}

int OSCEvent::OSC_writeStringArg(OSCbuf *buf, char *arg)
{
    int len;
    if (CheckTypeTag(buf, 's')) return 9;
    len = OSC_effectiveStringLength(arg);
    if (buf->gettingFirstUntypedArg && arg[0] == ',')
    {
        /* This un-type-tagged message starts with a string
        that starts with a comma, so we have to escape it
        (with a double comma) so it won't look like a type
        tag string. */

        CheckOverflow(buf, len+4); /* Too conservative */
        buf->bufptr += OSC_padStringWithAnExtraStupidComma(buf->bufptr, arg);
    }
    else
    {
        CheckOverflow(buf, len);
        buf->bufptr += OSC_padString(buf->bufptr, arg);
    }

    buf->gettingFirstUntypedArg = 0;
    return 0;
}

int OSCEvent::OSC_effectiveStringLength(char *string)
{
    int len = strlen(string) + 1;  /* We need space for the null char. */
    /* Round up len to next multiple of STRING_ALIGN_PAD to account for alignment padding */
    if ((len % STRING_ALIGN_PAD) != 0)
    {
        len += STRING_ALIGN_PAD - (len % STRING_ALIGN_PAD);
    }
    return len;
}

int OSCEvent::OSC_padString(char *dest, char *str)
{
    int i;
    for (i = 0; str[i] != '\0'; i++)
    {
        dest[i] = str[i];
    }
    return OSC_WritePadding(dest, i);
}

int OSCEvent::OSC_padStringWithAnExtraStupidComma(char *dest, char *str)
{
    int i;
    dest[0] = ',';
    for (i = 0; str[i] != '\0'; i++)
    {
        dest[i+1] = str[i];
    }
    return OSC_WritePadding(dest, i+1);
}

int OSCEvent::OSC_WritePadding(char *dest, int i)
{
    dest[i] = '\0';
    i++;
    for (; (i % STRING_ALIGN_PAD) != 0; i++)
    {
      dest[i] = '\0';
    }
    return i;
}

int OSCEvent::CheckOverflow(OSCbuf *buf, int bytesNeeded)
{
    if ((bytesNeeded) > OSC_freeSpaceInBuffer(buf))
    {
        SendError("buffer overflow");
        return 1;
    }
    return 0;
}

///////////////////////////////OSCTT/////////////////////////////////////

OSCTimeTag OSCEvent::OSCTT_CurrentTime(void)
{
    OSCTimeTag result;
    result.seconds = 0;
    result.fraction = 1;
    return result;
}

OSCTimeTag OSCEvent::OSCTT_BiggestPossibleTimeTag(void)
{
    OSCTimeTag result;
    result.seconds = 0xffffffff;
    result.fraction = 0xffffffff;
    return result;
}

OSCTimeTag OSCEvent::OSCTT_Immediately(void)
{
    OSCTimeTag result;
    result.seconds = 0;
    result.fraction = 1;
    return result;
}

OSCTimeTag OSCEvent::OSCTT_PlusSeconds(OSCTimeTag original, float secondsOffset)
{
    OSCTimeTag result;
    result.seconds = 0;
    result.fraction = 1;
    return result;
}

int OSCEvent::OSCTT_Compare(OSCTimeTag left, OSCTimeTag right)
{
    /* Untested! */
    int highResult = left.seconds - right.seconds;
    if (highResult != 0) return highResult;
    return left.fraction - right.fraction;
}
