/***************************************************************************
                          rtmix-parser.h  -  description
                             -------------------
    begin                : Wed Dec 5 2001
    copyright            : (C) 2001 by Ivica Bukvic
    email                : ico@fuse.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "eventcue.h"
#include "randfuncs.h"

#ifndef RTMIX_PARSER_H
#define RTMIX_PARSER_H

/*////////////////////////////////////////////////////////BEGIN PARSING FUNCTIONS/////////////////////////////////////////////////////////////////*/

QString RTMix::Do(QString s, QString tempS, QString error)
{
    int check, filterCheck;
    QString value, tempvalue;
    bool ok;
    QString OSCCheck, tmp_1, tmp_2, midiCheck, filterList, filter;
    int i, count, num;

	check = 1;

	if ( tempS.find("*[>do]") != - 1 ) { Error ( "\nWARNING! 'No error checking' flag set for 'do' command in line" + error + "...\n" ); check = 0; }//THIS MEANS THAT USER GIVES UP THE ERROR CHECK FOR THIS VALUE
	position = tempS.find( "[>do]" ) + 5;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks and brackets
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for ''do' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks and brackets
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for ''do' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	value = s.mid( startposition, endposition - startposition );
    tempvalue = value.lower();
    tempvalue = tempvalue.simplifyWhiteSpace();

    //check for the v[ ] and midi[ ] statements
    startposition = 0;
    while ( startposition != -1 )
    {
        startposition = tempvalue.find( "v[", startposition );
        if ( startposition != -1 )
        {
            //qDebug("Found v[");
            //make sure that it is lower-case
            value = value.replace( startposition, 1, "v" );
            endposition = tempvalue.find( "]", startposition);
            if ( endposition == -1 )
            {
                Error ( "\nError in line " + error + "... No closing ']' for the v[<num>] variable found in the given event...\n" );
                return "-1";
            }
            //qDebug( tempvalue.mid( startposition+2, endposition - (startposition + 2) ) );
            check = tempvalue.mid( startposition+2, endposition - (startposition + 2) ).toInt( &ok, 10 );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... v[<num>] variable is not a number...\n" );
                return "-1";
            }
            else if ( check < 0 || check > MAXVARS )
            {
                Error ( "\nError in line " + error + "... v[<num>] variable is outside range of 0-" + tmp_1.setNum(MAXVARS) + "...\n" );
                return "-1";
            }
        }
        if ( startposition >= 0 ) { startposition++; }
    }

    startposition = 0;
    while ( startposition != -1 )
    {
        startposition = tempvalue.find( "midi[", startposition );
        //make sure that it is lower-case
        if ( startposition != -1 )
        {
            //qDebug("Found midi[");
            value = value.replace( startposition, 4, "midi" );
            if ( TempCurr->accel.find("midi:",0) == -1 )
            {
                Error ( "\nError in line " + error + "... Midi variables (midi[ ]) found in the event statement, but there is no Midi accelerator present...\n" );
                return "-1";
            }
            endposition = tempvalue.find( "]", startposition );
            if ( endposition == -1 )
            {
                Error ( "\nError in line " + error + "... No closing ']' for the midi[<num>] variable found in the given event...\n" );
                return "-1";
            }
            //qDebug( tempvalue.mid( startposition + 5 , endposition - (startposition + 5) ) );
            check = tempvalue.mid( startposition + 5 , endposition - (startposition + 5) ).toInt( &ok, 10 );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... midi[<num>] variable is not a number...\n" );
                return "-1";
            }
            else if ( check < 0 || check > 2 )
            {
                Error ( "\nError in line " + error + "... midi[<num>] variable is outside range of 0-2...\n" );
                return "-1";
            }
        }
        if ( startposition >= 0 ) { startposition++; }
    }

            
    if (tempvalue != "none" && tempvalue.find("osc:", 0 ) == -1 && tempvalue.find("oscroute:", 0 ) == -1 && tempvalue.find("midisend:", 0 ) == -1 && tempvalue.find("osccustom:", 0 ) == -1 )
    {
        if (tempvalue.find("&", -1) == -1)
        {
            value = value + "&";
        }
        //FILTER EVENT AND PRODUCE WARNING
        filterList = S->GetFilter();
        filter = " ";
        filterCheck = 0;
        while ( filter != "" )
        {
            filter = filterList.section(",", filterCheck, filterCheck );
            //qDebug( filter );
            if ( value.contains( filter ) > 0 && filter != "" )
            {
                Error( "WARNING: your system-call event contains '" + filter + "' filtered event, and therefore it will not be executed at run-time. To alleviate this, either adjust your filters or your event..." );
            }
            filterCheck++;            
        }       
    }
    if ( tempvalue == "none" )
    {
        value == tempvalue; //Make sure that "none" is a lowercase statement
    }
    if ( tempvalue.find("osc:", 0 ) != -1 )
    {
        //qDebug("Found osc:");
        //OSC Event


        tempvalue = tempvalue.section(":", 1 ); //get rid of the initial OSC: stuff
        tempvalue = tempvalue.simplifyWhiteSpace(); //remove any initial spaces
        //Syntax is: host, port, tags, OSC_message,OSC_arg_1,OSC_arg_2
        //assuming that the host is ok
        //qDebug( tempvalue );
        OSCCheck =  tempvalue.section( ",", 1,1 );
        //qDebug( OSCCheck );
        num = OSCCheck.toInt( &ok, 10 );
        //qDebug ( "osc: %d" , num );
        if ( !ok )
        {
            Error ( "\nError in line " + error + "... OSC call has a PORT number that appears not to be a number (syntax: host, port, (bool)typetag, string, (8-bit unsigned) int, int)...\n" );
            value = "-1";
            return value;
        }
        OSCCheck =  tempvalue.section( ",", 2,2 );
        num = OSCCheck.toInt( &ok, 10 );
        //qDebug ( "osc: %d" , num );
        if ( !ok || num < 0 && num > 1 )
        {
            Error ( "\nError in line " + error + "... OSC call has a TYPETAG number that appears not to be of a boolean type (syntax: host, port, (bool)typetag, string, (8-bit unsigned) int, int)...\n" );
            value = "-1";
            return value;
        }

        tempvalue = tempvalue.section( ",", 4, -1 ); //get the values only
        //qDebug( "osc message stuff: " + tempvalue );
        i = 0;
        num = 0;

        //find how many parameters were given by numbering spaces
        while ( i != -1 )
        {
            i = tempvalue.find( ",", i+1 );
            if ( i > 0 )
            {
              num++;
            }
        }
        if ( num > 1 )
        {
             Error ( "\nError in line " + error + "... This implementation of OSC supports only sending maximum of 2 parameters at once (syntax: host, port, (bool)typetag, string, (8-bit unsigned) int, int)...\n" );
             return "-1";
        }

        for ( i =0; i <= num; i++ )
        {
            OSCCheck =  tempvalue.section( ",", i, i );
            num = OSCCheck.toInt( &ok, 10 );
            //qDebug ( "osc: %d" , num );
            if ( !ok || num < -1 )
            {
                Error ( "\nError in line " + error + "... OSC supports only sending MIDI-based data (unsigned Integers) and \
values greater than -1 (-1 being reserved for 'no data' situations)...\n" );
                return "-1";
            }
        }
    }
    if ( tempvalue.find("oscroute:", 0 ) != -1 )
    {
        if ( tempvalue.find("osc:", 0 ) != -1 )
        {
            Error ( "\nError in line " + error + "... only one kind of event can be associated with each event-call...\n" );
            return "-1";
        }
        //qDebug("Found oscroute:" );
        //OSCROUTE Event
        if ( TempCurr->accel == "none" )
        {
            Error ( "\nError in line " + error + "... OSCROUTE is used only for the REAL-TIME events and this one seems to be regular timed event (no [>accel] present)...\n" );
            return "-1";
        }
        else
        {
            if ( TempCurr->accel.find( "midi:", 0 ) == -1 )
            {
                Error ( "\nError in line " + error + "... OSCROUTE is used only in combination with MIDI -- this event does not use MIDI as an accelerator...\n" );
                value = "-1";
                return value;
            }
            
            //QString OSCCheck, tmp_1, tmp_2;
            //int i, num;
            //bool ok;

            tempvalue = tempvalue.section(":", 1 ); //get rid of the initial OSCROUTE: stuff
            tempvalue = tempvalue.simplifyWhiteSpace(); //remove any initial spaces
            //qDebug("oscroute: " + tempvalue );
            tempvalue = tempvalue.section(",", 1, -1 ); //get the values only (assume that the host is ok)
            i = 0;
            num = 0;
            //qDebug("oscroute: " + tempvalue );
            //find how many parameters were given by numbering commas
            while ( i != -1 )
            {
                i = tempvalue.find( ",", i+1 );
                //qDebug ( "i = %d", i );
                if ( i > 0 )
                {
                  num++;
                }
            }
            //qDebug ( "num = %d", num );
            if ( num != 1 )
            {
                Error ( "\nError in line " + error + "... Found OSCROUTE event that is not delineated properly -- Not enough parameters stated (syntax: host, port, (bool)typetags)...\n" );
                return "-1";
            }

            for ( i = 0; i < 2; i++ )
            {
                tmp_1 = tempvalue.section(",", i, i );
                num = tmp_1.toInt( &ok, 10 );
                //qDebug ( "oscroute: %d" , num );
                if ( !ok )
                {
                    Error ( "\nError in line " + error + "... Found OSCROUTE event that is not delineated properly -- parameter no. " + tmp_2.setNum(i + 2)  + "  is not a number (syntax: host, port, (bool)typetags)...\n" );
                    value = "-1";
                    return value;
                }
                if ( i == 1 && num < 0 || i == 1 && num > 1 )
                {
                    Error ( "\nError in line " + error + "... Found OSCROUTE event that is not delineated properly -- parameter no. " + tmp_2.setNum(i + 2)  + "  is not of a type boolean (syntax: host, port, (bool)typetags)...\n" );
                    return "-1";
                }
            }
        }
    }
    if ( tempvalue.find("midisend", 0 ) != -1 )
    {
        if ( tempvalue.find("osc:", 0 ) != -1 || tempvalue.find("oscroute:", 0 ) != -1 )
        {
            Error ( "\nError in line " + error + "... only one kind of event can be associated with each event-call...\n" );
            return "-1";
        }
        //qDebug("Found midisend");
        //MIDISEND Event
        //QString midiCheck, tmp_1;
        //int i, count, num;
        //bool ok;

        tempvalue = tempvalue.section(":", 1 ); //get rid of the initial OSC: stuff
        tempvalue = tempvalue.simplifyWhiteSpace(); //remove any initial spaces
        
        i = 0;
        count = 0;

        while ( i != -1 )
        {
            i = tempvalue.find( ",", i+1 );
            if ( i > 0 )
            {
              count++;
            }
        }        

         for( i = 0; i <= count; i++ )
        {
            midiCheck =  tempvalue.section( ",", i, i );
            num = midiCheck.toInt( &ok, 10 );
            //qDebug ( "midisend: %d", num );
            if ( !ok || num < 0 || num > 255 )
            {
                Error ( "\nError in line " + error + "... MIDISEND data no. " + tmp_1.setNum(i+1) + " is not a number or is not a valid 8-bit unsigned int...\n" );
                value = "-1";
                return value;
            }
        }
    }
    if ( tempvalue.find("osccustom:", 0 ) != -1 )
    {
        if ( tempvalue.find("osc:", 0 ) != -1 || tempvalue.find("oscroute:", 0 ) != -1 || tempvalue.find("midisend:", 0 ) != -1 )
        {
            Error ( "\nError in line " + error + "... only one kind of event can be associated with each event-call...\n" );
            return "-1";
        }
        //OSC CUSTOM EVENT
        tempvalue = tempvalue.section(":", 1 ); //get rid of the initial OSC: stuff
        tempvalue = tempvalue.simplifyWhiteSpace(); //remove any initial spaces
        //Syntax is: host, port, tags, OSC_message1 | OSC_message2 etc.
        //assuming that the host is ok
        OSCCheck =  tempvalue.section( ",", 1,1 );
        //qDebug( OSCCheck );
        num = OSCCheck.toInt( &ok, 10 );
        if ( !ok )
        {
            Error ( "\nError in line " + error + "... OSC_CUSTOM call has a PORT number that appears not to be a number (syntax: host, port, (bool)typetag, string, (8-bit unsigned) int, int)...\n" );
            value = "-1";
            return value;
        }
        OSCCheck =  tempvalue.section( ",", 2,2 );
        num = OSCCheck.toInt( &ok, 10 );
        if ( !ok || num < 0 && num > 1 )
        {
            Error ( "\nError in line " + error + "... OSC_CUSTOM call has a TYPETAG number that appears not to be of a boolean type (syntax: host, port, (bool)typetag, string, (8-bit unsigned) int, int)...\n" );
            value = "-1";
            return value;
        }
        Error( "WARNING: No syntax checking will be done on the parameters of the OSC_CUSTOM due to lack of a standard. Please make sure to use pipe '|' for separating the messages within a bundle.\n" );
    }
    return value;
}

long RTMix::At(QString s, QString tempS, QString error)
{
    QString value, tmp_1;
    long time;
    bool ok;
    float num;


    position = tempS.find("[>at]") + 5;
    startposition = s.find ( "{\"", position ) + 2;
    if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
        Error ( "\nError in line " + error + "... Insufficient event data provided for 'at' command (check your quotation marks and brackets)...\n" );
        return -1;
    }
    endposition = s.find( "\"}", startposition );
    if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
        Error ( "\nError in line " + error + "... Insufficient event data provided for 'at' command (check your quotation marks and brackets)...\n" );
        return -1;
    }
    value = s.mid( startposition, endposition - startposition );
    num = value.toFloat( &ok );
    if ( !ok )
    {
        if ( value.simplifyWhiteSpace().find( "rt[" ) == 0 )
        {
            //FOUND A REAL-TIME EVENT
            value = value.mid( 3, value.length() - 4 );
            //qDebug( value );
            time = value.toInt( &ok, 10 );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... 'at' real-time value is not a number...\n" );
                return -1;
            }
            time = -time - 1000; //this way we differ from RT events and regular timed events
            if ( time < -1000 - MAXRTEVENTS || time > - 1000 )
            {
                Error ( "\nError in line " + error + "... Real-time event number out of avaliable range of 0-" + tmp_1.setNum(MAXRTEVENTS) + "...\n" );
            }
            //qDebug ( "%i", int(time) );
            if ( TempCurr->accel == "none" )
            {
                Error ( "\nError in line " + error + "... 'at' value reflects real-time binding, but there is no corresponding [>accel] value found...\n" );
                return -1;
            }
            return time;
        }
        else
        {
            Error ( "\nError in line " + error + "... Ambiguous 'at' value entered...\n" );
            return -1;
        }
    }
    if ( TempCurr->accel != "none" )
    {
        Error ( "\nError in line " + error + "... Event is ambiguous: 'at variable implies NON-real-time binding, but the event also includes [>accel] value...\n" );
        return -1;
    }
    time = long( (value.toFloat( 0 ) + 0.005) * 100 );
    if ( time > 35999999 || time < 0 ) {
        Error ( "\nError in line " + error + "... Time value out of Timer's range...\n" );
        return -1;
    }
    return time;
}

QString RTMix::Accel(QString s, QString tempS, QString error)
{
	QString value;
    int low, high;

	position = tempS.find( "[>accel]" ) + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
 		Error ( "\nError in line " + error + "... Insufficient event data provided for 'accel' command (check your quotation marks and brackets)...\n" );
 		value = "-1";
 		return value;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
 		Error ( "\nError in line " + error + "... Insufficient event data provided for 'accel' command (check your quotation marks and brackets)...\n" );
 		value = "-1";
 		return value;
	}
	value = s.mid( startposition, endposition - startposition );

    //qDebug( value );
    //Check for MIDI accels:
    if (value.lower().find( "midi:", 0 ) != -1 )
    {
        Error( "Found MIDI accelerator...\n" );
        //qDebug("Found MIDI accel" );
        QString midiCheck, tmp_1, tmp_2, tmp_3;
        int i, num;
        bool ok;

        midiCheck = value.section(":", 1); //get only stuff after the MIDI: instruction
        midiCheck = midiCheck.simplifyWhiteSpace();
        //qDebug( midiCheck );
        /*
        if ( midiCheck == value )
        {
            Error ( "\nError in line " + error + "... Found MIDI instruction that is not delineated properly, use 'MIDI:' (with column) before listing the parameters...\n" );
            value = "-1";
            return value;
        }
        */
        //midiCheck = midiCheck.section(",", 1, -1 ); //take only numbers (assume host is good no matter what and report any potential errors at runtime)
        num = 0;
        i = 0;

        //find how many parameters were given by numbering commas
        while ( i != -1 )
        {
            i = midiCheck.find( ",", i+1 );
            if ( i > 0 )
            {
              num++;
            }
            //qDebug( "%d, %d", i, num );
        }
        //qDebug( "midiaccel , = %d", num );
        if ( num != 5 )
        {
            Error ( "\nError in line " + error + "... Found MIDI accelerator that is not delineated properly: you need 6 parameteres (syntax: (8-bit unsigned int)low-status, high-status, low-command1, high-command1, low-command2, high-command2)...\n" );
            return "-1";
        }
        
        low = 0;
        high = 0;

        for ( i = 0; i < 6; i++ )
        {
            tmp_1 = midiCheck.section(",", i, i );
            num = tmp_1.toInt( &ok, 10 );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... Found MIDI accelerator that is not delineated properly: parameter no. " + tmp_2.setNum(i+1)  + "  is not a number (syntax: (8-bit unsigned int)low-status, high-status, low-command1, high-command1, low-command2, high-command2)...\n" );
                return "-1";
            }
            if ( i == 0 || i == 2 || i == 4 )
            {
                low = num;
            }
            else
            {
                high = num;
                if ( high < low )
                {
                    Error ( "\nError in line " + error + "... MIDI accelerator's parameter no. " + tmp_2.setNum(i+1) + " cannot be smaller than the parameter no. " + tmp_3.setNum( i ) + "...\n" );
                    return "-1";
                }
            }
            if ( i < 4 && num < 0 || i >= 4 && num < -1 || num > 255 )
            {
                if ( i < 2 )
                {
                    Error ( "\nError in line " + error + "... MIDI accelerator's parameter no. " + tmp_2.setNum(i+1) + " is outside allowed range 0-255...\n" );
                }
                else
                {
                     Error ( "\nError in line " + error + "... MIDI accelerator's parameter no. " + tmp_2.setNum(i+1) + " is outside allowed range -1-255 (-1 for midi events that might not require the second data)...\n" );
                }
                return "-1";
            }
            switch( i )
            {
                case 0:
                    TempCurr->la = num;
                    break;
                case 1:
                    TempCurr->ha = num;
                    break;
                case 2:
                    TempCurr->lb = num;
                    break;
                case 3:
                    TempCurr->hb = num;
                    break;
                case 4:
                    TempCurr->lc = num;
                    break;
                case 5:
                    TempCurr->hc = num;
                    break;                 
            }
            TempCurr->last1 = -1;
            TempCurr->last2 = -1;
            TempCurr->last3 = -1;
            TempCurr->lastTime = -5;
            //qDebug ( "MIDI-debug: %d", num );
        }
    }
    else
    {
        if ( value.length() > 1 && value.length() != 2 || value.length() == 2 && !value.startsWith("^") )
        {
            Error ( "\nError in line " + error + "... Found ambiguous keyboard accelerator that is longer than one character...\n" );
            return "-1";
        }
    }
    return value;
}

QString RTMix::Text(QString s, QString tempS, QString error)
{
	int check;
	QString value, tempvalue;

	check = 1;

	if ( tempS.find("*[>say]") != - 1 ) { Error ( "WARNING! 'No error checking' flag set for 'say' command in line" + error + "..." ); check = 0; }//THIS MEANS THAT USER GIVES UP THE ERROR CHECK FOR THIS VALUE
	position = tempS.find( "[>say]" ) + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'say' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'say' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	value = s.mid( startposition, endposition - startposition );
	TempCurr->say = value;
	if ( value.lower() == "none" )
	{
		return "none";
	}
	tempvalue = value.simplifyWhiteSpace();
	tempvalue = tempvalue.lower();
	if ( tempvalue.find( "<font", 0 ) == -1 && tempvalue.find( "< font", 0 ) == -1 )
	{
     	value = "<font color=\"#" + DefaultTextInterfaceColor + "\">" + value + "</font>";
	}
	return value;
}

QString RTMix::Kill(QString s, QString tempS, QString error)
{
	int check;
	QString value;

	check = 1;

	if ( tempS.find("*[>kill]") != - 1 ) { Error ( "WARNING! 'No error checking' flag set for 'kill' command in line" + error + "..." ); check = 0; }//THIS MEANS THAT USER GIVES UP THE ERROR CHECK FOR THIS VALUE
	position = tempS.find( "[>kill]" ) + 7;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		if (check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'kill' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		if (check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'kill' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	value = s.mid( startposition, endposition - startposition );
	return value;
}

long RTMix::Dur(QString s, QString tempS, QString error)
{
	QString value;
	long time;

	position = tempS.find("[>dur]") + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'dur' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'dur' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	time = long( (value.toFloat( 0 ) + 0.005) * 100 );
	if ( time > 35999999 || time < 0 ) {
		Error ( "\nError in line " + error + "... Time value out of Timer's range...\n" );
			return -1;
	}
	return time;
}

long RTMix::Chance(QString s, QString tempS, QString error)
{
	QString value;
	long chance;
    float tmp;
    bool ok;

	position = tempS.find("[>chance]") + 9;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'chance' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'chance' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
    tmp = value.toFloat( &ok );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'chance' value is outside the valid range (0-1)...\n" );
        return -1;
    }
	chance = 10000 - long( (tmp + 0.005) * 10000 ); //chance is measured as a long number, 10000 being equal to 100.00%

	return chance;
}

long RTMix::Happen(QString s, QString tempS, QString error)
{
	QString value;
	long happen;
    float tmp;
    bool ok;
    
	position = tempS.find("[>happen]") + 9;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'happen' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'happen' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
    tmp = value.toFloat( &ok );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'happen' value is outside the valid range (0-1)...\n" );
        return -1;
    }
	happen = long( (tmp + 0.005) * 10000 ); //happen is measured as a long number, 10000 being equal to 100.00%

	return happen;
}

QString RTMix::Color(QString s, QString tempS, QString error)
{
	int check, position;
	QString value, test;

	check = 1;

	position = tempS.find( "[>color]" ) + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		if (check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'color' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		if (check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'color' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	value = s.mid( startposition, endposition - startposition );
    test = tempS.mid( startposition, endposition - startposition );
	test = test.simplifyWhiteSpace();
	
	if ( TempCurr->text != "none" )
	{
		if ( TempCurr->text.find("font color", 0) == -1 )
		{	
			//Error( "Found no font info, inserting default...\n"); //DEBUG
			TempCurr->text = "<font color = \"#" + value + "\">" + TempCurr->text + "</font>";
		}
		else
		{
         	position = TempCurr->text.find("\"#", 0) + 2;
			TempCurr->text.replace( position, 6, value );
		}		
	}
	
	return value;
}

int RTMix::Light(QString s, QString tempS, QString error)
{
	QString value;
	int light;
    bool ok;

	position = tempS.find("[>light]") + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'light' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'light' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	light = int( (value.toFloat( &ok ) + 0.005) );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'light' value is not a number...\n" );
        return -1;
    }
	if ( light > 13 || light < 0 ) {
		Error ( "\nError in line " + error + "... 'light' value out of available range...\n" );
			return -1;
	}
	return light;
}

int RTMix::BlinkSpeed(QString s, QString tempS, QString error)
{
	QString value;
	int var;
    bool ok;

	position = tempS.find("[>blinkspeed]") + 13;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'blinkspeed' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'blinkspeed' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( &ok ) + 0.005) * 100 );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'blinkspeed' value is not a number...\n" );
        return -1;
    }
	if ( var > 1000 || var < 10 ) {
		Error ( "\nError in line " + error + "... 'blinkspeed' value out of available range of 0.1 and 10.00 seconds...\n" );
			return -1;
	}
	return var;
}

int RTMix::LightLength(QString s, QString tempS, QString error)
{
	QString value;
	int var;
    bool ok;

	position = tempS.find("[>lightlength]") + 14;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'lightlength' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'lightlength' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( &ok ) + 0.005) * 100 );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'lightlength' value is not a number...\n" );
        return -1;
    }
	if ( var > 1000 || var < 0 ) {
		Error ( "\nError in line " + error + "... 'lightlength' value out of available range of 10.00 seconds...\n" );
			return -1;
	}
	return var;
}

int RTMix::prepLength(QString s, QString tempS, QString error)
{
	QString value;
	int var;
    bool ok;

	position = tempS.find("[>preplength]") + 13;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'preplength' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'preplength' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( &ok ) + 0.005) * 100 );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'preplength' value is not a number...\n" );
        return -1;
    }
	if ( var > 6000 || var < 0 ) {
		Error ( "\nError in line " + error + "... 'preplength' value out of available range of 60.00 seconds...\n" );
			return -1;
	}
	return var;
}

int RTMix::NetAddress(QString s, QString tempS, QString error)
{
	QString value, tmp_1;
	int var;

	position = tempS.find("[>net]") + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'net' command (check your quotation marks and brackets)...\n" );
		return -3;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'net' command (check your quotation marks and brackets)...\n" );
		return -3;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( 0 ) + 0.005) );
	if ( var > MAXNET || var < -2 ) {
		Error ( "\nError in line " + error + "... 'net' value out of available range of -2 to " + tmp_1.setNum(MAXNET) + " ( -2 for localhost ( NOT via network ), -1 for all external clients, and 0-" + tmp_1 + " for individual ones defined in the table )...\n" );
		return -3;
	}
	return var;
}

int RTMix::Bpm(QString s, QString tempS, QString error)
{
	QString value;
	int var;
    bool ok;

	position = tempS.find("[>bpm]") + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'bpm' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'bpm' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( &ok ) + 0.005) );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'bpm' value is not a number...\n" );
        return -1;
    }
	if ( var > 999 || var < 0 ) {
		Error ( "\nError in line " + error + "... 'bpm' value out of available range of 0-999...\n" );
			return -1;
	}
	return var;
}

int RTMix::MetNum(QString s, QString tempS, QString error)
{
	QString value;
	int var;
	bool ok;

	position = tempS.find("[>num]") + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'num' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'num' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok, 10 );
	if ( !ok )
	{
     	Error ( "\nError in line " + error + "... Entry for the 'num' value is not a number...\n" );
		return -1;
	}
	if ( var < 1 && var > 4 )
	{
     	Error ( "\nError in line " + error + "... Entry for the 'num' value is outside of the 1-4 range...\n" );
		return -1;
	}
	return var;
}

QString RTMix::Meter(QString s, QString tempS, QString error)
{
    QString value, check;
    bool ok;
    int position, oldPosition, test, num;

    position = tempS.find("[>meter]") + 8;
    startposition = s.find ( "{\"", position ) + 2;
    if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
        Error ( "\nError in line " + error + "... Insufficient event data provided for 'meter' command (check your quotation marks and brackets)...\n" );
        return "-1";
    }
    endposition = s.find( "\"}", startposition );
    if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
        Error ( "\nError in line " + error + "... Insufficient event data provided for 'meter' command (check your quotation marks and brackets)...\n" );
        return "-1";
    }
    value = s.mid( startposition, endposition - startposition );
    //TODO: (optional) check the meter's integrity
    position = 0;
    oldPosition = 0;

    num = 0;
    
    if ( value.find("+", position ) == -1 ) //ONE ELEMENT ONLY
    {
        test = value.toInt( &ok, 10 );
        if ( !ok )
        {
            Error ( "\nError in line " + error + "...  'meter' value is not a number...\n" );
            return "-1";
        }
        else if ( test == 0 )
        {
            Error ( "\nError in line " + error + "...  'meter' cannot be 0...\n" );
            return "-1";
        }
    }
    else while ( position != -1 )
    {
        position = value.find("+", oldPosition + 1 );
        if ( oldPosition == 0 )
        {
            check = value.mid( oldPosition, position );
        }
        else
        {
            check = value.mid( oldPosition + 1, position - (oldPosition + 1) );
        }
        test = check.toInt( &ok, 10 );
        if ( !ok )
        {
            Error ( "\nError in line " + error + "... One (or more) of the 'meter' values are not a number (make sure your syntax is int+int+int+etc.)...\n" );
            return "-1";
        }
        if ( position != -1 )
        {
            oldPosition = position;
            num++;
            if ( num > 49 )
            {
                Error ( "\nError in line " + error + "... Maximum of 50 subdivisions of a meter are allowed...\n" );
                return "-1";
            }
        }
    }
    return value;
}

QString RTMix::Jump(QString s, QString tempS, QString error)
{
	QString value;
	int var;
	bool ok;

	position = tempS.find( "[>to]" ) + 5;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'to' command (check your quotation marks and brackets)...\n" );
		return "error";
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'to' command (check your quotation marks and brackets)...\n" );
		return "error";
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok, 10 );
	if ( !ok )
	{
    	Error( "\nError in line " + error + "... 'to' expression is not a number...\n" );
		return "error"; 	
	}
	return value;
}

QString RTMix::Amount(QString s, QString tempS, QString error)
{
	QString value;
	float var;
	bool ok;

	position = tempS.find( "[>amt]" ) + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'amt' command (check your quotation marks and brackets)...\n" );
		return "error";
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'amt' command (check your quotation marks and brackets)...\n" );
		return "error";
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toFloat( &ok );
	if ( !ok )
	{
    	Error( "\nError in line " + error + "... 'amt' expression has a value that is not a number...\n" );
		return "error"; 	
	}
	return value;
}

QString RTMix::Func(QString s, QString tempS, QString error)
{
	int check;
	QString value, tempvalue, tmp_1;
    bool ok;

	check = 1;

	position = tempS.find( "[>func]" ) + 7;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'func' command (check your quotation marks and brackets)...\n" );
			return "-1";
		}
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'func' command (check your quotation marks and brackets)...\n" );
			return "-1";
		}
	}
	value = s.mid( startposition, endposition - startposition );
    tempvalue = value.lower();
    tempvalue = tempvalue.simplifyWhiteSpace();

    startposition = 0;
    //check for the v[ ] and midi[ ] statements
    while ( startposition != -1 )
    {
        startposition = tempvalue.find( "v[", startposition );
        if ( startposition != -1 )
        {
            //qDebug("Found v[");
            //make sure that it is lower-case
            value = value.replace( startposition, 1, "v" );
            endposition = tempvalue.find( "]", startposition);
            if ( endposition == -1 )
            {
                Error ( "\nError in line " + error + "... No closing ']' for the v[<num>] variable found in the given event...\n" );
                return "-1";
            }
            //qDebug( tempvalue.mid( startposition+2, endposition - startposition+2 ) );
            check = tempvalue.mid( startposition+2, endposition - (startposition+2) ).toInt( &ok, 10 );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... v[<num>] variable is not a number...\n" );
                return "-1";
            }
            if ( ok && check < 0 || ok && check > MAXVARS )
            {
                Error ( "\nError in line " + error + "... v[<num>] variable is outside range of 0-" + tmp_1.setNum(MAXVARS) + "...\n" );
                return "-1";
            }
        }
        if ( startposition >= 0 ) { startposition++; }
    }

    startposition = 0;

    while ( startposition != -1 )
    {
        startposition = tempvalue.find( "midi[", startposition );
        //make sure that it is lower-case
        if ( startposition != -1 )
        {
            //qDebug("Found midi[");
            value = value.replace( startposition, 4, "midi" );
            if ( startposition != -1 && TempCurr->accel.find("midi",0) == -1 )
            {
                Error ( "\nError in line " + error + "... Midi variables (midi[ ]) found in the event statement, but there is no Midi accelerator present...\n" );
                return "-1";
            }
            endposition = tempvalue.find( "]", startposition );
            if ( endposition == -1 )
            {
                Error ( "\nError in line " + error + "... No closing ']' for the midi[<num>] variable found in the given event...\n" );
                return "-1";
            }
            //qDebug( tempvalue.mid( startposition + 5 , endposition - startposition + 5 ) );
            check = tempvalue.mid( startposition + 5 , endposition - (startposition + 5) ).toInt( &ok, 10 );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... midi[<num>] variable is not a number...\n" );
                return "-1";
            }
            if ( ok && check < 0 || ok && check > 2 )
            {
                Error ( "\nError in line " + error + "... midi[<num>] variable is outside range of 0-2...\n" );
                return "-1";
            }
        }
        if ( startposition >= 0 ) { startposition++; }
    }
	return value;
}

QString RTMix::Var(QString s, QString tempS, QString error)
{
	int check;
	QString value, tempvalue, tmp_1;
    bool ok;

	check = 1;

	position = tempS.find( "[>var]" ) + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'var' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		if ( check ) {
			Error ( "\nError in line " + error + "... Insufficient event data provided for 'var' command (check your quotation marks and brackets)...\n" );
			value = "-1";
			return value;
		}
	}

	//FINAL CHECK FOR THE CORRESPONDING VALUE
	value = tempS.mid( startposition, endposition - startposition );
	value = value.lower();
	if ( value.find("v[",0) == -1 && value != "value" && value != "abs" && value != "meter" && value != "event"  && value != "at" && value != "dur" && value != "absat" && value != "happen" && value !="chance" && value != "color" && value != "light" && value != "blinkspeed" && value != "store" && value !="status" && value != "amt" && value != "preplength" && value != "lightlength" && value != "net"  && value != "bpm" && value != "type" && value != "method" && value != "num" && value != "to" && value != "id" )
	{
	    Error ( "\nError in line " + error + "... Could not parse the 'store' variable (or variable cannot have reassigned value)...\n" );
		return "-1";
	}
    tempvalue = value.simplifyWhiteSpace();
    startposition = tempvalue.find( "v[", startposition );
    if ( startposition != -1 )
    {
       //qDebug("Found v[");
       //make sure that it is lower-case
       value = value.replace( startposition, 1, "v" );
       endposition = tempvalue.find( "]", startposition);
       if ( endposition == -1 )
       {
           Error ( "\nError in line " + error + "... No closing ']' for the v[<num>] variable found in the 'store' parameter...\n" );
           return "-1";
       }
       //qDebug( tempvalue.mid( startposition+2, endposition - (startposition + 2) ) );
       check = tempvalue.mid( startposition+2, endposition - (startposition + 2) ).toInt( &ok, 10 );
       if ( !ok )
       {
           Error ( "\nError in line " + error + "... 'store' parameter's v[<num>] variable is not a number...\n" );
           return "-1";
       }
       else if ( check < 0 || check > MAXVARS )
       {
           Error ( "\nError in line " + error + "... 'store' parameter's v[<num>] variable is outside range of 0-" + tmp_1.setNum( MAXVARS ) + "...\n" );
           return "-1";
       }
   }
	return value;
}

int RTMix::Method(QString s, QString tempS, QString error)
{
	QString value;
	int var;
	bool ok;

	position = tempS.find("[>method]") + 9;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'method' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'method' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok, 10 );
	if ( !ok )
	{
     	Error ( "\nError in line " + error + "... Entry for 'method' is not a number...\n" );
		return -1;
	}
	if ( var > 6 || var <  0 ) {
		Error ( "\nError in line " + error + "... 'method' value out of available range of 0-6 ( rrand, brrand, gaussian, cauchy, linlo, linhi, and triangle )...\n" );
			return -1;
	}
	return var;
}

int RTMix::Type(QString s, QString tempS, QString error)
{
	QString value;
	int var;
    bool ok;

	position = tempS.find("[>type]") + 7;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'type' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'type' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok );
	if ( var > 65535 || var <  0 ) {
       Error ( "\nError in line " + error + "... 'type' value out of available range of -1 (-1 for float, 0 < x < 65536 for scaled int within that range)...\n" );
       return -1;
	}
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'type' value is not an integer...\n" );
        return -1;
    }
	return var;
}

int RTMix::ID(QString s, QString tempS, QString error)
{
	QString value, temp, tmp_1;
	bool ok;
    int val;

	position = tempS.find("[>id]") + 5;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'id' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'id' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	if ( value == "0" )
	{
     	return 0;
	}
	if ( value == "event" || value == "1" )
	{
     	return 1;
	}
	if ( value == "countdown" || value == "2" )
	{
     	return 2;
	}
	if ( value == "text" || value == "3" )
	{
     	return 3;
	}
	if ( value == "checkpoint" || value == "4" )
	{
     	return 4;
	}
	if ( value == "pause" || value == "5" )
	{
     	return 5;
	}
	if ( value == "stop" || value == "6" )
	{
     	return 6;
	}
	if ( value == "clear" || value == "7" )
	{
     	return 7;
	}
	if ( value == "warning" || value == "8" )
	{
     	return 8;
	}
	if ( value == "metronome" || value == "9" )
	{
     	return 9;
	}
	if ( value == "jump" || value == "10" )
	{
     	return 10;
	}
	if ( value == "randomize" || value == "11" )
	{
     	return 11;
	}
	if ( value == "change" || value == "12" )
	{
     	return 12;
	}
	if ( value == "seed" || value == "13" )
	{
     	return 13;
	}
	if ( value == "rew" || value == "14" )
	{
     	return 14;
	}
	if ( value == "ffw" || value == "15" )
	{
     	return 15;
	}
	if ( value == "togglemetro" || value == "16" )
	{
     	return 16;
	}
	if ( value == "assign" || value == "17" )
	{
     	return 17;
	}
	if ( value == "togglert" || value == "18" )
	{
     	return 18;
	}
	if ( value == "setnet" || value == "19" )
	{
     	return 19;
	}
	if ( value == "togglenet" || value == "20" )
	{
     	return 20;
	}
	if ( value == "togglemidi" || value == "21" )
	{
     	return 21;
	}

	//REAL-TIME MODIFIERS
	if ( value.find("rt[") != -1 )
	{
		value = value.mid( 3, value.length() - 4 );
        if ( value.find( "v[",0 ) == -1 )
        {
            int time = value.toInt( &ok, 10 );
            //qDebug( "rt[] %d" , time );
            if ( !ok )
            {
                Error ( "\nError in line " + error + "... Real-time event number is not a number...\n" );
                return -1;
            }
            time = -time - 1000; //this way we differ from RT events and regular timed events
            if ( time < -1000 - MAXRTEVENTS || time > - 1000 )
            {
                Error ( "\nError in line " + error + "... Real-time event number out of avaliable range of 0-" + tmp_1.setNum( MAXRTEVENTS ) + "...\n" );
            }
            //qDebug ( "RTevent %i", time );
            return time;
        }
        else
        {
            position = value.find( "v[", 0 );
            if ( position != -1 )
            {
                endposition = value.find( "]", position + 2 );
                temp = value.mid( position + 2, endposition - (position + 2) );
                //qDebug( "rt[v[]] " + temp );
                val = temp.toInt( &ok, 10 );

                if ( !ok )
                {
                    Error(  " -- Error parsing the v[<num>] value of the 'id' entry...\n" );
                    return -1;
                }
                else if ( ok && val < 0 || ok && val > MAXVARS )
                {
                    Error(  " -- Error parsing the v[<num>] value of the 'id' entry: value outside allowed range (0-" + tmp_1.setNum( MAXVARS ) + ")...\n" );
                    return -1;
                }
                else
                {
                    if ( position >= 0 )
                    {
                        return -val - 3000;
                    }
                }
            }
        }
	}
   position = value.find( "v[", 0 );
   if ( position != -1 )
   {
       endposition = value.find( "]", position + 2 );
       temp = value.mid( position + 2, endposition - (position + 2) );
       //qDebug( "v[] id" + temp );
       val = temp.toInt( &ok, 10 );

       if ( !ok )
       {
           Error(  " -- Error parsing the v[<num>] value of the 'id' entry...\n" );
           return -1;
       }
       else if ( ok && val < 0 || ok && val > totalEvents )
       {
           Error(  " -- Error parsing the v[<num>] value of the 'id' entry: value outside allowed range (0-" + temp.setNum( totalEvents, 10 ) + ")...\n" );
           return -1;
       }
       else
       {
           if ( position >= 0 )
           {
               //qDebug( "test %d", -val - 2000 );
               return -val - 2000;
           }
       }
   }
	//QString t;
	Error ( "\nError in line " + error + "... 'type' value out of available range of 0-" + temp.setNum( totalEvents, 10 ) + " (see manual for detailed listing of each event's id)...\n" );
	return -1;
}

int RTMix::RandVar(QString s, QString tempS, QString error)
{
	QString value, tmp_1;
	int var;

	position = tempS.find("[>store]") + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'var' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'var' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( 0 ) + 0.005) );
	if ( var > MAXVARS || var <  0 ) {
		Error ( "\nError in line " + error + "... 'var' value out of available range of 0-" + tmp_1.setNum( MAXVARS ) + "...\n" );
			return -1;
	}
	return var;
}

float RTMix::Amt(QString s, QString tempS, QString error)
{
	QString value;
	float var;
	bool ok;

	position = tempS.find("[>amt]") + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'amt' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'amt' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toFloat( &ok );
	if ( !ok )
	{
		Error ( "\nError in line " + error + "... Entry for 'amt' is not a number...\n" );
	}
	return var;
}

QString RTMix::Abs(QString s, QString tempS, QString error)
{
	QString value;
	int var;

	position = tempS.find("[>abs]") + 6;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'abs' command (check your quotation marks and brackets)...\n" );
		return "-1";
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'abs' command (check your quotation marks and brackets)...\n" );
		return "-1";
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( 0 ) + 0.005) );
	if ( var > 1 || var <  0 ) {
		Error ( "\nError in line " + error + "... 'type' value out of available range of 0-1 (0 for relative, 1 for absolute)...\n" );
		return "-1";
	}
	return value;
}

long RTMix::SeedType(QString s, QString tempS, QString error)
{
	QString value;
	long var;
	bool ok;

	position = tempS.find("[>value]") + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'value' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'value' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toLong( &ok, 10 );
	if ( !ok )
	{
     	Error( "\nError in line " + error + "... Entry for 'value' is not a long integer number...\n" );
		return -1;
	}
	if ( var < 0 ) {
     	Error( "\nError in line " + error + "... Entry for 'value' cannot be less than zero...\n" );
		return -1;		
	}
	return var;
}

int RTMix::MetroStatus(QString s, QString tempS, QString error)
{
	QString value;
	int var;

	position = tempS.find("[>status]") + 9;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'status' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'status' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( 0 ) + 0.005) );
	if ( var > 3 || var <  0 ) {
		Error ( "\nError in line " + error + "... 'status' value out of available range of 0-3 (0 for stop&reset, 1 for stop, 2 for start&reset, and 3 for start )...\n" );
		return -1;
	}
	return var;
}

int RTMix::RTStatus(QString s, QString tempS, QString error)
{
	QString value;
	int var;

	position = tempS.find("[>status]") + 9;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'status' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'status' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( 0 ) + 0.005) );
	if ( var > 1 || var <  0 ) {
		Error ( "\nError in line " + error + "... 'status' value out of available range of 0-1 (0 for stop, 1 for start )...\n" );
		return -1;
	}
	return var;
}

int RTMix::Absat(QString s, QString tempS, QString error)
{
	QString value;
	int var;
    bool ok;

	position = tempS.find("[>absat]") + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'absat' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'absat' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = int( (value.toFloat( &ok ) + 0.005) );
    if ( !ok )
    {
        Error ( "\nError in line " + error + "... 'absat' value is not a number...\n" );
        return -1;
    }
	if ( var > 1 || var <  0 ) {
		Error ( "\nError in line " + error + "... 'absat' value out of available range of 0-1 (0 for 'false' and 1 for 'true' )...\n" );
			return -1;
	}
	return var;
}

QString RTMix::Host(QString s, QString tempS, QString error)
{
	QString value;

	position = tempS.find( "[>host]" ) + 7;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
 		Error ( "\nError in line " + error + "... Insufficient event data provided for 'host' command (check your quotation marks and brackets)...\n" );
 		value = "-1";
 		return value;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
 		Error ( "\nError in line " + error + "... Insufficient event data provided for 'host' command (check your quotation marks and brackets)...\n" );
 		value = "-1";
 		return value;
	}
	value = s.mid( startposition, endposition - startposition );
	return value;
}

int RTMix::Port(QString s, QString tempS, QString error)
{
	QString value;
	int var;
	bool ok;

	position = tempS.find("[>socket]") + 9;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'socket' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'socket' command (check your quotation marks and brackets)...\n" );
		return -1;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok, 10 );
	if ( var < 0 || !ok ) {
		Error ( "\nError in line " + error + "... 'socket' value not a number, or less than 0...\n" );
		return -1;
	}
	return var;
}

int RTMix::NetID(QString s, QString tempS, QString error)
{
	QString value, tmp_1;
	int var;
	bool ok;

	position = tempS.find("[>id]") + 5;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'id' command (check your quotation marks and brackets)...\n" );
		return -3;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'id' command (check your quotation marks and brackets)...\n" );
		return -3;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok, 10 );
	if ( var < -2 || !ok || var > MAXNET ) {
		Error ( "\nError in line " + error + "... 'id' value not a number, or not in range of -2-" + tmp_1.setNum( MAXNET ) + " (-2 for local socket, -1 for 'all' external connections, and 0-" + tmp_1 + " for respective client connections)...\n" );
		return -3;
	}
	return var;
}

int RTMix::NetStore(QString s, QString tempS, QString error)
{
	QString value, tmp_1;
	int var;
	bool ok;

	position = tempS.find("[>store]") + 8;
	startposition = s.find ( "{\"", position ) + 2;
	if (startposition == -1 || s.find( "[>", position ) < startposition  && s.find( "[>", position ) != -1 || s.find( ");;", position ) < startposition && s.find( ");;", position ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'store' command (check your quotation marks and brackets)...\n" );
		return -3;
	}
	endposition = s.find( "\"}", startposition );
	if (endposition == -1 || s.find( "[>", startposition ) < endposition  && s.find( "[>", startposition ) != -1 || s.find( ");;", startposition ) < endposition && s.find( ");;", startposition ) != -1 ) { //if there is no closing quotation marks
		Error ( "\nError in line " + error + "... Insufficient event data provided for 'store' command (check your quotation marks and brackets)...\n" );
		return -3;
	}
	value = s.mid( startposition, endposition - startposition );
	var = value.toInt( &ok, 10 );
	if ( var < -2 || !ok || var > MAXNET ) {
		Error ( "\nError in line " + error + "... 'id' value not a number, or not in range of -2-" + tmp_1.setNum (MAXNET) + " (-2 for localhost ( NOT via network ), -1 for 'all', and 0-" + tmp_1 + " for respective connections)...\n" );
		return -3;
	}
	return var;
}


/*////////////////////////////////////////////////////////END PARSING FUNCTIONS///////////////////////////////////////////////////////////////////*/


int RTMix::CheckOptional(QString s, QString tempS, QString error)
{
	if ( tempS.find( "[>chance]" ) != -1 )
	{
		TempCurr->chance = Chance(s, tempS, error);
		if ( TempCurr->chance < 0 )
		{
			return -1;
		}
		//TempCurr->happen = 10000;
	}

	if ( tempS.find( "[>happen]" ) != -1 )
	{
		TempCurr->happen = Happen(s, tempS, error);
		if ( TempCurr->happen < 0 )
		{
			return -1;
		}
	}

	if ( tempS.find( "[>color]" ) != -1 )
	{
		TempCurr->color = Color(s, tempS, error);
		if ( TempCurr->color == "-1" )
		{
			return -1;
		}
	}

	if ( tempS.find( "[>light]" ) != -1 )
	{
		TempCurr->light = Light(s, tempS, error);
		if ( TempCurr->light < 0 )
		{
			return -1;
		}
	}

	if ( tempS.find( "[>blinkspeed]" ) != -1 )
	{
		TempCurr->blinkSpeed = BlinkSpeed(s, tempS, error);
		if ( TempCurr->blinkSpeed < 0 )
		{
			return -1;
		}
	}

	if ( tempS.find( "[>lightlength]" ) != -1 )
	{
		TempCurr->lightLength = LightLength(s, tempS, error);
		if ( TempCurr->lightLength < 0 )
		{
			return -1;
		}
	}

	if ( tempS.find( "[>preplength]" ) != -1 )
	{
		TempCurr->prepLength = prepLength(s, tempS, error);
		if ( TempCurr->prepLength < 0 )
		{
			return -1;
		}
	}

	if ( tempS.find( "[>net]" ) != -1 && TempCurr->netAddress != -3 )
	{
     	Error( "\nWARNING: 'net' value is not applicable to this event. Please consult the manual for more info. Ignoring 'net' value...\n" );
		CompileComments = true;
	}
	if ( tempS.find( "[>net]" ) != -1 && TempCurr->netAddress == -3 )
	{
		TempCurr->netAddress = NetAddress(s, tempS, error);
		if ( TempCurr->netAddress < 0 )
		{
			return -1;
		}
	}	

	if ( TempCurr->light > 3 && tempS.find("[>dur]") != -1 ) {
		TempCurr->countdown = Dur(s, tempS, error);
		if ( TempCurr->countdown < 0 )
		{
			return -1;
		}
	}

	//FINAL CHECK TO SEE IF EVERYTHING IS SANE:
    /*
	if ( TempCurr->light == 3 && TempCurr->countdown == 0 )  //FOR CUSTOM EVENTS THAT DO REQUIRE COUNTDOWN-LIKE VISUALS
	{
		Error("\nWARNING: Due to fact you are using 'countdown' display settings for a non-countdown object, you need to specify [>dur] value for the duration of the countdown.");
		if ( TempCurr->prepLength == 0 )
		{
         	Error(" Also, if you wish to have blinking light as the countdown approaches zero, use [>preplength] to specify the amount of time...\n" );
		}
		else
		{
         	Error("..\n");
		}
		TempCurr->countdown = Dur(s, tempS, error);
		if ( TempCurr->countdown < 0 )
		{
			return -1;
		}	
    }
    */
    if ( TempCurr->light > 3 && TempCurr->light < 8 && TempCurr->countdown > 5999 )
    {
        Error( "Error in line " + error + "... [>light] value reflects use of a 'warning' window, but the [>dur] value exceeds its maximum displayable duration (59.99 seconds)...\n" );
        return -1;
        
    }
	if ( TempCurr->countdown != 0 && TempCurr->light < 3 && TempCurr->light > 0 )
	{
     	TempCurr->lightLength = TempCurr->countdown;
		CompileComments = true;
		Error("\nWARNING: [>dur] value used for [>lightlength] due to fact it reflects the true duration of the event...\n");
	}
	if ( TempCurr->light == 2 && TempCurr->blinkSpeed == 0 )
	{
		CompileComments = true;
	 	Error("\nWARNING: [>light] value is set to 2, but [>blinkspeed] is 0, this will make the light to behave like the light mode 1. Therefore, the [>light] setting has been reverted to 1...\n");
		TempCurr->light = 1;
	}
	if ( TempCurr->light > 3 && TempCurr->countdown != 0 && TempCurr->prepLength == 0 )
	{
		Error("\nWARNING: use [>preplength] to specify the amount of blinking time before the 'execute' signal...\n" );
	}
    if ( TempCurr->ID == 4 && TempCurr->accel != "none" || TempCurr->ID == 4 && TempCurr->time < -10 )
    {
        Error ( "Error in line " + error + "... 'checkpoint' event cannot be bound to a real-time event...\n" );
        return -1;
    }
    if ( TempCurr->ID == 22 && TempCurr->time >= 0 )
    {
        Error ( "Error in line " + error + "... 'play' event must be bound to a real-time event...\n" );
        return -1;
    }
	if ( TempCurr->ID == 7 && TempCurr->text != "none" )
	{
		Error("WARNING: use of [>say] parameter has no effect on the 'clear' event. Ignoring the parameter...\n" );
	}
	if ( TempCurr->absat == 1 && TempCurr->accel != "none" )
	{
		Error("WARNING: use of [>absat] parameter has no effect on the real-time event. Ignoring the parameter...\n" );
	}
	if ( TempCurr->prepLength != 0 && TempCurr->countdown == 0 )
	{
		Error("WARNING: use of [>preplength] parameter has no effect on this event since 'dur' is equal to 0. Ignoring the parameter...\n" );
	}
	return 0;
}

















int RTMix::Compile()
{
    if ( PlaybackStatus > 0 ) {
        Error( "Compiling impossible during playback!\n" );
        return 0;
    }

    if ( !ErrorCompile )
    {
        Stop( NULL ); //reset the counter
    }

    ErrorCompile = TRUE;
    T->ToggleAErrors( FALSE );
    T->ToggleAWarns( FALSE );

    Error( "Disabling network activity and purging network info...\n" );
    PurgeNetwork();

    RTMix::PurgeCue();
    CompileComments = false;

    int found, good, check;
    QString test;
    bool ok;
  
    QString OSCHost, TmpEvent, TmpName;
    bool OSCTags;
    int OSCPort;
    QColor CellColor;
    //int OSCParams[6];
    //char *OSCMessage;
    QString RTDescription;
    
	//network stuff
	QString tmp, host;
	int sck, store;

	Error( "Compiling...\n" );
	QString o, s, tempS, error, value, column;
	o = Editor->text();
	QTextStream t( &o, IO_ReadOnly );

	line = 0;
	nEvents = 0; //Resetting number of compiled events

	while ( !t.eof() ) {
		found = 0; //RESET "FOUND RECOGNIZABLE COMMAND"
		good = 0; //RESET "THE LINE IS GOOD TO BE STORED INTO THE CUE" VALUE
		check = 1; //RESET "CHECK FOR ERRORS" FLAG

		//DO NOT NEED CHECK VAR!!!!!

		line++;
       error.setNum ( line, 10 );

		s = t.readLine();
		tempS = s.lower();
		//s = s.simplifyWhiteSpace();
		if ( s.find( "//", 0 ) == -1 && s.simplifyWhiteSpace() != "" ) {

			TempCurr = new eventCue;
			TempCurr->next = NULL;
			TempCurr->prev = NULL;

          TempCurr->prepLength = 0; //DEFAULT FOR THE COUNTDOWN SETTINGS
          TempCurr->bpm = 0; //ONLY METRO USES IT SO MAY AS WELL SET IT TO 0
          TempCurr->text = "none";
          TempCurr->say = "none";
          TempCurr->absat = 0;
          TempCurr->type = 0; //ONLY RANDOMIZE USES IT
          TempCurr->var = "";	//ONLY CHANGE USES IT
          TempCurr->method = 0; //ONLY RANDOMIZE USES IT
          TempCurr->amt = 0.0; // USED BY ASSIGN
          TempCurr->killEvent = "none";
          TempCurr->accel = "none";
          TempCurr->OSCPtr = NULL;
          TempCurr->time = -10;

          TempCurr->la = -1;
          TempCurr->ha = -1;
          TempCurr->lb = -1;
          TempCurr->hb = -1;
          TempCurr->lc = -1;
          TempCurr->hc = -1;

          TempCurr->last1 = -10;
          TempCurr->last2 = -10;
          TempCurr->last3 = -10; //last used midi var in order to prevent overcrowding sys calls (one per 5 centiseconds)
          TempCurr->lastTime = -10;
          
			//FIND OPTIONAL TEXT
			if ( tempS.find( "[>say]" ) != -1 ) {
				TempCurr->text = Text(s, tempS, error);
               	if ( TempCurr->text == "-1" )
				{
					delete TempCurr;
					return 0;
				}					
			}

			//FIND OPTIONAL ABSAT CALL -- reflects whether calls respond to absolute or relative (main) timer [ABSAT meaning Absolute At]
			if ( tempS.find( "[>absat]" ) != -1 ) {
				TempCurr->absat = Absat(s, tempS, error);
               	if ( TempCurr->absat == -1 )
				{
					delete TempCurr;
					return 0;
				}					
			}

			//ACCELERATOR value applicable for all events, but used only by the rt-events
			if ( tempS.find( "[>accel]" ) != -1 ) {
				TempCurr->accel = Accel(s, tempS, error);
              if ( TempCurr->accel == "-1" )
				{
					delete TempCurr;
					return 0;
				}    
			}

 			//FIND TIMING
 			if ( tempS.find( "[>at]" ) != -1 ) {
 				TempCurr->time = At(s, tempS, error);
 				if ( TempCurr->time == -1 )
 				{
 					delete TempCurr;
 					return 0;
 				}
 				found++;
 			}
      
			//IF FOUND AN EVENT=========================================================
			if ( tempS.find( "event", 0, false ) == 0 ) {
				Error( "found an 'event' in line " + error + "..." ); //DEBUG
				TempCurr->ID = 1; //EVENT HAS AN ID 1

				//FIND ACTION
				if ( tempS.find( "[>do]" ) != -1 ) {
					TempCurr->event = Do(s, tempS, error);
                	if ( TempCurr->event == "-1" )
					{
						delete TempCurr;
						return 0;
					}
					found++;  //increment every found element so that parser can know how many elements for the event are currently present

					//INSTANTIATE OSCevent CLASS AS NEEDED
                  if ( TempCurr->event.lower().find("oscroute:", 0) != -1 || TempCurr->event.lower().find("osccustom:", 0) != -1 || TempCurr->event.lower().find("osc:", 0) != -1 )
                  {
                     //QString OSCHost, TmpEvent, TmpMessage;
                     //bool OSCTags;
                     //int OSCPort, OSCLoop;
                     //int OSCParams[6];
                     //char *OSCMessage;

                     TmpEvent = TempCurr->event.section(":", 1);
                     TmpEvent = TmpEvent.simplifyWhiteSpace();
                     OSCHost = TmpEvent.section(",", 0,0);
                     //qDebug( "Host = " + OSCHost );
                     OSCPort = TmpEvent.section(",", 1,1).toInt( NULL, 10 ); //stuff has been checked so no need to recheck it
                     //qDebug("Port to be sent to OSCEvent %d", OSCPort );
                     OSCTags = TmpEvent.section(",", 2,2).toInt( NULL, 10 );
                     //qDebug("time = %d", TempCurr->time );
                     TmpName.setNum( (-(TempCurr->time) - 999), 10 );  //incrementing by one the name so that it corresponds to the table names
                     //TmpMessage = TmpEvent.section(",",3);
                     //TmpMessage = TmpMessage.simplifyWhiteSpace();
                     //OSCMessage = const_cast<char *>( TmpMessage.latin1() );
                     //OSCMessage = (char *)TmpEvent.section(",", 3);
                     //for ( OSCLoop = 0; OSCLoop < 6; OSCLoop++ )
                     //{
                     //   OSCParams[OSCLoop] = TmpEvent.section(",",OSCLoop+4).toInt(NULL);
                     //}
                     //qDebug( TmpName );
                     TempCurr->OSCPtr = new OSCEvent( this, TmpName, 0, OSCHost, OSCPort, OSCTags, "null",
                            TempCurr->la, TempCurr->ha, TempCurr->lb, TempCurr->hb, TempCurr->lc, TempCurr->hc );
                     if ( TempCurr->OSCPtr->GetStatus() == -1 )
                     {
                          Error( "\nError trying to create a socket for the OSC -- please check your network connection. Aborting further compilation...\n" );
                          delete TempCurr;
                          return 0;
                     }
                     else
                     {
                        haveOSC = true;
                     }
                     //qDebug( "created oscroute..." );
                  }
				}

				//FIND KILL
				if ( tempS.find( "[>kill]" ) != -1 ) {
					TempCurr->killEvent = Kill(s, tempS, error);
                	if ( TempCurr->killEvent == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some necessary parameters for the 'event' are missing ( check if you have [>event], [>at], and [>kill] )...\n" );
					delete TempCurr;
					return 0;
				}

				good = 1;
				TempCurr->countdown = 0;           //Not being used by the event

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -3;				

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END EVENT		
			}





			//IF FOUND A COUNTDOWN=========================================================
			if ( tempS.find( "countdown", 0, false ) == 0 ) {
				TempCurr->ID = 2; //COUNTDOWN HAS AN ID 2
				Error( "found a 'countdown' in line " + error + "..." );   //DEBUG

				//FIND COUNTDOWN DATA (to be executed at the end of the countdown)
				if ( tempS.find( "[>do]" ) != -1 ) {
					TempCurr->event = Do(s, tempS, error);
                	if ( TempCurr->event == "-1" )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				//FIND KILL
				if ( tempS.find( "[>kill]" ) != -1 ) {
					TempCurr->killEvent = Kill(s, tempS, error);
                	if ( TempCurr->killEvent == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//FIND COUNTDOWN  VALUE
				if ( tempS.find( "[>dur]" ) != -1 ) {
					TempCurr->countdown = Dur(s, tempS, error);
					if ( TempCurr->countdown < 0 )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				if ( found < 4 )
				{
					Error ( "\nError in line " + error + "... Some parameters for the 'countdown' are missing ( check if you have [>do], [>at], [>dur], and [>kill] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 3;						//Light
				TempCurr->lightLength = 200;		//default
				TempCurr->prepLength = 500;      //default
				TempCurr->blinkSpeed = 100;		//default
				TempCurr->netAddress = -3;					

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1)
				{
					delete TempCurr;
					return 0;
				}

			//END COUNTDOWN
			}



			//IF FOUND A TEXT EVENT=========================================================
			if ( s.find( "text", 0, false ) == 0 ) {
				Error( "found a 'text' event in line " + error + "..." );   //DEBUG
				TempCurr->ID = 3; //TEXT HAS AN ID 3

				//FIND TEXT
				if ( tempS.find( "[>say]" ) != -1 ) {
					TempCurr->text = Text(s, tempS, error);
                	if ( TempCurr->text == "-1" )
					{
						delete TempCurr;
						return 0;
					}					
					found++;
				}

				if ( found < 2 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'text' are missing ( check if you have [>at] and [>say] )...\n" );
					delete TempCurr;
					return 0;
				}

				good = 1;
				TempCurr->countdown = 0;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR -------- TODO: ASSIGN IT THE VALUE OF THE DISPLAY WIDGET COLOR, ZERO MAKES IT INVISIBLE
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -3;

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END TEXT EVENT
			}



			//IF FOUND A CHECKPOINT=========================================================
			if ( s.find( "checkpoint", 0, false ) == 0 ) {
				Error( "found a 'checkpoint' in line " + error + "..." );    //DEBUG
				TempCurr->ID = 4; //CHECKPOINT HAS AN ID 4

				if ( found < 1 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'checkpoint' are missing ( check if you have [>at] )...\n" );
					delete TempCurr;
					return 0;
				}

				good = 1;
				TempCurr->countdown = 0;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR -------- TODO: ASSIGN IT THE VALUE OF THE DISPLAY WIDGET COLOR, ZERO MAKES IT INVISIBLE
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2;			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}
        
			//END CHECKPOINT
			}







			//IF FOUND "PAUSE" COMMAND=========================================================
			if ( s.find( "pause", 0, false ) == 0 ) {
				Error( "found 'pause' command in line " + error + "..." );   //DEBUG
				TempCurr->ID = 5; //PAUSE HAS AN ID 5

				if ( found < 1 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'pause' are missing ( check if you have [>at] )...\n" );
					delete TempCurr;
					return 0;
				}

				good = 1;
				TempCurr->countdown = 0;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR -------- TODO: ASSIGN IT THE VALUE OF THE DISPLAY WIDGET COLOR, ZERO MAKES IT INVISIBLE
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -1; 			//all only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END PAUSE
			}







			//IF FOUND "STOP" COMMAND=========================================================
			if ( s.find( "stop", 0, false ) == 0 ) {
				Error( "found 'stop' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 6; //PAUSE HAS AN ID 6

				if ( found < 1 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'stop' are missing ( check if you have [>at] )...\n" );
					delete TempCurr;
					return 0;
				}
				
				good = 1;
				TempCurr->countdown = 0;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR -------- TODO: ASSIGN IT THE VALUE OF THE DISPLAY WIDGET COLOR, ZERO MAKES IT INVISIBLE
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -1; 			//all only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END STOP
			}








			//IF FOUND "CLEAR" COMMAND=========================================================
			if ( s.find( "clear", 0, false ) == 0 ) {
				Error( "found 'clear' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 7; //CLEAR HAS AN ID 7

				if ( found < 1 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'clear' are missing ( check if you have [>at] )...\n" );
					delete TempCurr;
					return 0;
				}

				good = 1;
				TempCurr->countdown = 0;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR -------- TODO: ASSIGN IT THE VALUE OF THE DISPLAY WIDGET COLOR, ZERO MAKES IT INVISIBLE
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -3;

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END CLEAR
			}




			//IF FOUND "WARNING" COMMAND=========================================================
			if ( s.find( "warning", 0, false ) == 0 ) {
				Error( "found 'warning' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 8; //WARNING HAS ID 8

				//FIND WARNING DATA (to be executed at the begnning of the warning)
				if ( tempS.find( "[>do]" ) != -1 ) {
					TempCurr->event = Do(s, tempS, error);
                	if ( TempCurr->event == "-1" )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				//FIND KILL
				if ( tempS.find( "[>kill]" ) != -1 ) {
					TempCurr->killEvent = Kill(s, tempS, error);
                	if ( TempCurr->killEvent == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//FIND COUNTDOWN  VALUE
				if ( tempS.find( "[>dur]" ) != -1 ) {
					TempCurr->countdown = Dur(s, tempS, error);
					if ( TempCurr->countdown < 0 )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				if ( found < 4 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'warning' are missing ( check if you have [>do], [>at], [>dur], and [>kill] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 2;						//Light is not functional by default for the event
				TempCurr->lightLength = 200;		//Light is on by default
				TempCurr->blinkSpeed = 50;			//Blinking by default
				TempCurr->netAddress = -3;				

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END WARNING
			}










			//IF FOUND "METRONOME" COMMAND=========================================================
			if ( s.find( "metro", 0, false ) == 0 ) {
				Error( "found 'metro' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 9; //METRONOME HAS ID 9

				//FIND BPM (stored in the "bpm" variable)
				if ( tempS.find( "[>bpm]" ) != -1 ) {
					TempCurr->bpm = Bpm(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				//FIND METRONOME IDENTIFIER (1-4)
				if ( tempS.find( "[>num]" ) != -1 ) {
					TempCurr->type = MetNum(s, tempS, error);
                	if ( TempCurr->type == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//FIND NUMBER OF BEATS/MEASURE
				if ( tempS.find( "[>meter]" ) != -1 ) {
					TempCurr->event = Meter(s, tempS, error);
					if ( TempCurr->event == "-1" )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				if ( found < 4 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'metro' are missing ( check if you have [>at], [>num], [>meter], and [>bpm] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -3;

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END METRONOME
			}









			//IF FOUND "JUMP" COMMAND=========================================================
			if ( s.find( "jump", 0, false ) == 0 ) {
				Error( "found 'jump' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 10; //JUMP HAS ID 10

				//JUMP LOCATION
				if ( tempS.find( "[>to]" ) != -1 ) {
					test = Jump(s, tempS, error);
                		if ( test == "error" )
					{
						delete TempCurr;
						return 0;
					}
					TempCurr->bpm = test.toInt( &ok, 10 );
					//qDebug( "COMPILING -- Time: %i Jump: %i", TempCurr->time, TempCurr->bpm );	
					found++;
				}

				//ABSOLUTE OR RELATIVE
				if ( tempS.find( "[>abs]" ) != -1 ) {
					TempCurr->event = Abs(s, tempS, error);
                	if ( TempCurr->event == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				if ( TempCurr->bpm < 0 && TempCurr->event == "1" )
				{
                  Error ( "WARNING: 'jump' event is absolute, but the 'to' value is negative. Interpreting 'to' value as a positive number..." );
                 	TempCurr->bpm = abs( TempCurr->bpm );
				}

				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'jump' are missing ( check if you have [>at], [>abs], and [>to] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;
				TempCurr->killEvent = "none";
				//if ( TempCurr->absat != 0 )
				//{
                	// 	Error( "\n'Absat' command cannot be used for jump, rew, or ffw events since the Absolute timer is not affected by them. Resetting 'absat' to its default value of 0...\n" );
				//		TempCurr->absat = 0;
				//}

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -4; 			//global by default

				if ( TempCurr->bpm < 0 && TempCurr->event == "1" )
				{
					Error ( "\nWARNING: Adjusting 'to' value to its absolute value due to fact that Jump was requested as absolute, while value given was less than 0...\n" );
					CompileComments = true;
				}

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END JUMP
			}









			//IF FOUND "RANDOMIZE" COMMAND=========================================================
			if ( s.find( "randomize", 0, false ) == 0 ) {
				Error( "found 'randomize' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 11; //RANDOMIZE HAS ID 11

				//VAR
				if ( tempS.find( "[>store]" ) != -1 ) {
					TempCurr->bpm = RandVar(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//TYPE (0 for float, 0 < x < 65536 for scaled int)
				if ( tempS.find( "[>type]" ) != -1 ) {
					TempCurr->type = Type(s, tempS, error);
                	if ( TempCurr->type == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//TYPE (0 for int, 1 for float)
				if ( tempS.find( "[>method]" ) != -1 ) {
					TempCurr->method = Method(s, tempS, error);
                	if ( TempCurr->method == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				if ( found < 4 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'randomize' are missing ( check if you have [>at], [>type], [>method], and [>store] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2; 			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END RANDOMIZE
			}







			//IF FOUND "CHANGE" COMMAND=========================================================
			if ( s.find( "change", 0, false ) == 0 ) {
				Error( "found 'change' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 12;

				//FUNCTION
				if ( tempS.find( "[>func]" ) != -1 ) {
					TempCurr->event = Func(s, tempS, error);
                	if ( TempCurr->event == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//VAR
				if ( tempS.find( "[>var]" ) != -1 ) {
					TempCurr->var = Var(s, tempS, error);
                	if ( TempCurr->var == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//Optional EVENT description if more than one change-event is to affect the same event
				if ( tempS.find( "[>id]" ) != -1 ) {
					TempCurr->type = ID(s, tempS, error);
                		if ( TempCurr->type == -1 )
					{
						delete TempCurr;
						return 0;
					}
				}
				else
				{
                 		TempCurr->type = 0;
				}

				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'change' are missing ( check if you have [>at], [>func], and [>var] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2; 			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END CHANGE
			}








			//IF FOUND "SEED" COMMAND=========================================================
			if ( s.find( "seed", 0, false ) == 0 ) {
				Error( "found 'seed' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 13; //REPEAT HAS ID 13

				//FUNCTION TYPE
				if ( tempS.find( "[>value]" ) != -1 ) {
					TempCurr->amt = SeedType(s, tempS, error);
                	if ( TempCurr->amt == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				if ( found < 2 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'seed' are missing ( check if you have [>at] and [>value] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";

				TempCurr->happen = 10000;			//100% chance of happening
				TempCurr->chance = 0;			//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2; 			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END SEED
			}








			//IF FOUND "REW" COMMAND=========================================================
			if ( s.find( "rew", 0, false ) == 0 ) {
				Error( "found 'rew' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 14;

				//JUMP LOCATION
				if ( tempS.find( "[>to]" ) != -1 ) {
					test = Jump(s, tempS, error);
                	if ( test == "error" )
					{
						delete TempCurr;
						return 0;
					}
					TempCurr->bpm = test.toInt( &ok, 10 );	
					found++;
				}

				if ( found < 2 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'ffw' are missing ( check if you have [>at] and [>to] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";
/*
				if ( TempCurr->absat != 0 )
				{
                 	Error( "\n'Absat' command cannot be used for jump, rew, or ffw events since the Absolute timer is not affected by them. Resetting 'absat' to its default value of 0...\n" );
					TempCurr->absat = 0;
				}
*/
				if ( TempCurr->bpm < 0 && TempCurr->event == "1" )
				{
                  Error ( "WARNING: 'to' value should be always positive. Interpreting 'to' value as a positive number..." );
                 	TempCurr->bpm = abs( TempCurr->bpm );
				}

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -4; 			//global only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END REW
			}










			//IF FOUND "FFW" COMMAND=========================================================
			if ( s.find( "ffw", 0, false ) == 0 ) {
				Error( "found 'ffw' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 15;

				//JUMP LOCATION
				if ( tempS.find( "[>to]" ) != -1 ) {
					test = Jump(s, tempS, error);
                	if ( test == "error" )
					{
						delete TempCurr;
						return 0;
					}
					TempCurr->bpm = test.toInt( &ok, 10 );	
					found++;
				}

				if ( found < 2 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'ffw' are missing ( check if you have [>at] and [>to] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;
				TempCurr->event = "none";
				TempCurr->killEvent = "none";
/*
				if ( TempCurr->absat != 0 )
				{
                 	Error( "\n'Absat' command cannot be used for jump, rew, or ffw events since the Absolute timer is not affected by them. Resetting 'absat' to its default value of 0...\n" );
					TempCurr->absat = 0;
				}
*/
				if ( TempCurr->bpm < 0 && TempCurr->event == "1" )
				{
                  Error ( "WARNING: 'to' value should be always positive. Interpreting 'to' value as a positive number..." );
                 	TempCurr->bpm = abs( TempCurr->bpm );
				}

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -4; 			//global only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END FFW
			}









			//IF FOUND "TOGGLEMETRO" COMMAND=========================================================
			if ( s.find( "togglemetro", 0, false ) == 0 ) {
				Error( "found 'togglemetro' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 16;

				//METRO ID
				if ( tempS.find( "[>num]" ) != -1 ) {
					TempCurr->type = MetNum(s, tempS, error);
                	if ( TempCurr->type == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//STATUS
				if ( tempS.find( "[>status]" ) != -1 ) {
					TempCurr->bpm = MetroStatus(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'togglemetro' are missing ( check if you have [>at], [>status], and [>num] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;
				TempCurr->event = "none";

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -3;

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END TOGGLEMETRO
			}






			//IF FOUND "ASSIGN" COMMAND=========================================================
			if ( s.find( "assign", 0, false ) == 0 ) {
				Error( "found 'assign' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 17;

				//VAR
				if ( tempS.find( "[>store]" ) != -1 ) {
					TempCurr->bpm = RandVar(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//AMOUNT
				if ( tempS.find( "[>amt]" ) != -1 ) {
					test = Amount(s, tempS, error);
                	if ( test == "error" )
					{
						delete TempCurr;
						return 0;
					}
					TempCurr->amt = test.toFloat( &ok );	
					found++;
				}

				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'assign' are missing ( check if you have [>at], [>store], and [>amt] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2; 			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END ASSIGN
			}








			//IF FOUND "TOGGLERT" COMMAND=========================================================
			if ( s.find( "togglert", 0, false ) == 0 ) {
				Error( "found 'togglert' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 18;

				//STATUS
				if ( tempS.find( "[>status]" ) != -1 ) {
					TempCurr->bpm = RTStatus(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				if ( found < 2 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'togglert' are missing ( check if you have [>at] and [>status] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2;			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END TOGGLERT
			}






			//IF FOUND "SETNET" COMMAND=========================================================
			if ( s.find( "setnet", 0, false ) == 0 ) {
				Error( "found 'setnet' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 19;

              if ( TempCurr->time != -10 ) //due to fact timing is now a generic "search" while setnet does not need it
              {
                  TempCurr->time = -10;
                  found--;
                  Error ( "WARNING: setnet in line " + error + " does not need the [>at] value. Ignoring the value...\n");
              }

				//HOST
				if ( tempS.find( "[>host]" ) != -1 ) {
					host = Host(s, tempS, error);
                	if ( host == "-1" )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//SOCKET
				if ( tempS.find( "[>socket]" ) != -1 ) {
					sck = Port(s, tempS, error);
                	if ( sck == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//STORE IN
				store = -1;
				if ( tempS.find( "[>store]" ) != -1 ) {
 					store = NetStore(s, tempS, error);
               		if ( store == -3 )
 					{
 						delete TempCurr;
 						return 0;
 					}
					found++;	
				}
				if ( host == "localhost" && store == -1 || host == "127.0.0.1" && store == -1 )
				{
					found++;
					Error ( "WARNING: treating event in line " + error + " as a socket-changing event...\n");
					CompileComments = true;
				}


				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'setnet' are missing ( check if you have [>host], [>socket] ( if specifying external network connection ), and [>store] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				if ( host == "localhost" && store == -1 )
				{
					Error ( "Adjusting localhost's socket..." );
					if ( sck != socket )
					{
						ChangeLocalSocket( sck );
					}
				}
				else
				{
					Error ( "Updating network table..." );
                 	ExternalClientsTable->setText( store, 0, host );
                 	ExternalClientsTable->setText( store, 1, tmp.setNum( sck, 10 ) );
					ExternalClientsTable->setText( store, 2, "OFF" );
					net[store] = new Client( this, tmp.setNum( store, 10 ), host, sck );
					connect( net[store], SIGNAL ( EmitText( QString ) ), this, SLOT ( Error( QString ) ) );
				}

			//END SETNET
			}




			//IF FOUND "TOGGLENET" COMMAND=========================================================
			if ( s.find( "togglenet", 0, false ) == 0 ) {
				Error( "found 'togglenet' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 20;

				//STATUS
				if ( tempS.find( "[>status]" ) != -1 ) {
					TempCurr->bpm = RTStatus(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}	
					found++;
				}

				//NETWORK CONNECTION TO BE AFFECTED
				if ( tempS.find( "[>id]" ) != -1 ) {
					TempCurr->type = NetID(s, tempS, error);
                	if ( TempCurr->type == -3 )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				if ( found < 3 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'togglenet' are missing ( check if you have [>at], [>id], and [>status] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2; 			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END TOGGLENET
			}






			//IF FOUND "TOGGLEMIDI" COMMAND=========================================================
			if ( s.find( "togglemidi", 0, false ) == 0 ) {
				Error( "found 'togglemidi' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 21;


				//STATUS
				if ( tempS.find( "[>status]" ) != -1 ) {
					TempCurr->bpm = RTStatus(s, tempS, error);
                	if ( TempCurr->bpm == -1 )
					{
						delete TempCurr;
						return 0;
					}
					found++;
				}

				if ( found < 2 ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'togglemidi' are missing ( check if you have [>at] and [>status] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -2;			//local only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END TOGGLEMIDI
			}



      

			//IF FOUND "PLAY" COMMAND=========================================================
			if ( s.find( "play", 0, false ) == 0 ) {
				Error( "found 'play' command in line " + error + "..." );  //DEBUG
				TempCurr->ID = 22;

				if ( found < 1 || TempCurr->accel == "none" ) {
					Error ( "\nError in line " + error + "... Some parameters for the 'togglemidi' are missing ( check if you have [>at] and [>accel] )...\n" );
					delete TempCurr;
					return 0;
				}
				good = 1;

				TempCurr->happen = 10000;					//100% chance of happening
				TempCurr->chance = 0;					//100% hidden chance number for changing purposes
				TempCurr->color = "0";					//STANDARD COLOR
				TempCurr->light = 0;						//Light is not functional by default for the event
				TempCurr->lightLength = 0;			//Light is off by default
				TempCurr->blinkSpeed = 0;			//No blinking by default
				TempCurr->netAddress = -1;			//all only

				//NOW CHECK FOR THE OPTIONAL STUFF
                if (CheckOptional(s, tempS, error) == -1) {
					delete TempCurr;
					return 0;
				}

			//END PLAY
			}

      



			//FINAL ERROR CHECKING===================================================================
			//IF THERE IS MORE THAN ONE COMMAND PER LINE
			column = s.simplifyWhiteSpace();
			if ( column.find( ");;" ) != column.find ( ");;", -3 ) || column.find( ");;" ) == -1 ) {
				Error ( "\nError in line " + error + "... Only one command per line allowed or the closing ';;' are not properly set...\n" );
				delete TempCurr;
				return 0;
			}
			//IF NONE OF THE KNOWN COMMANDS ARE FOUND, SOUND AN ERROR
			if ( !found ) {
				Error ( "\nError in line " + error + "... Unknown command...\n" );
				delete TempCurr;
				return 0;
			}

			//PARSING OF THE REAL-TIME EVENTS
			if ( TempCurr->time < -10 )
			{
             	int store = -(TempCurr->time) - 1000;
				//qDebug( "Storing into %i", store );
				if ( RTEvents[ store ] != NULL )
				{
                 	delete RTEvents[ store ];
				}
				RTEvents[ store ] = new eventCue;
				RTEventsCopy[ store ] = new eventCue;
				*RTEvents[ store ] = *TempCurr;
				*RTEventsCopy[ store ] = *TempCurr;
				//Figure out coloring for a particular item
              //Default RT Event
              CellColor.setRgb( 170, 244, 110 );
              //Midi Event
              if ( RTEvents[store]->accel.lower().find( "midi:", 0 ) != -1 )
              {               
                  CellColor.setRgb( 226, 164, 242 );
                  RTDescription = "Midi trigger -> ";
              }
              else if ( RTEvents[store]->accel.startsWith("^") )
              {
                  RTDescription = "Keyboard Release -> ";
              }
              else
              {
                  RTDescription = "Keyboard Press -> ";
              }
              //OSC Route Event
              if ( RTEvents[store]->event.lower().find( "oscroute:", 0 ) != -1 )
              {
                  RTDescription = "OSC Router to " + (QString) RTEvents[store]->OSCPtr->hostname + ":" + tmp.setNum( RTEvents[store]->OSCPtr->portname, 10 );
                  CellColor.setRgb( 153, 195, 247 );
              }
              else if ( RTEvents[store]->event.lower().find("midisend:", 0) != -1 )
              {
                  RTDescription = RTDescription + "Midi event-array";
              }
              else if ( RTEvents[store]->event.lower().find( "osc:", 0 ) != -1 )
              {
                  RTDescription = RTDescription + "Single OSC event";
              }
              else if ( RTEvents[store]->event.lower().find( "osccustom:", 0 ) != -1 )
              {
                  RTDescription = RTDescription + "Single OSC_CUSTOM event";
              }
              else
              {
                  switch( RTEvents[store]->ID )
                  {
                      case 1:
                          RTDescription = RTDescription + "System call: " + RTEvents[store]->event;
                          break;
                      case 2:
                          RTDescription = RTDescription + "Countdown Event";
                          break;
                      case 3:
                          RTDescription = RTDescription + "Text Event";
                          break;
                      case 4:
                          RTDescription = RTDescription + "Checkpoint Event";
                          break;
                      case 5:
                          RTDescription = RTDescription + "Pause Event";
                          break;
                      case 6:
                          RTDescription = RTDescription + "Stop Event";
                          break;
                      case 7:
                          RTDescription = RTDescription + "Clear Event";
                          break;
                      case 8:
                          RTDescription = RTDescription + "Warning Event";
                          break;
                      case 9:
                          RTDescription = RTDescription + "Metronome Event";
                          break;
                      case 10:
                          RTDescription = RTDescription + "Jump Event";
                          break;
                      case 11:
                          RTDescription = RTDescription + "Randomize Event";
                          break;
                      case 12:
                          RTDescription = RTDescription + "Change Event";
                          break;
                      case 13:
                          RTDescription = RTDescription + "Seed Event";
                          break;
                      case 14:
                          RTDescription = RTDescription + "Rew Event";
                          break;
                      case 15:
                          RTDescription = RTDescription + "Ffw Event";
                          break;
                      case 16:
                          RTDescription = RTDescription + "ToggleMetro Event";
                          break;
                      case 17:
                          RTDescription = RTDescription + "Assign Event";
                          break;
                      case 18:
                          RTDescription = RTDescription + "ToggleRt Event";
                          break;
                      case 19:
                          RTDescription = RTDescription + "SetNet Event";
                          break;
                      case 20:
                          RTDescription = RTDescription + "ToggleNet Event";
                          break;
                      case 21:
                          RTDescription = RTDescription + "ToggleMidi Event";
                          break;                     
                  }
              }
				RTEventTable->PostText( store, 0, CellColor, s.mid( 0, s.find("(") )  );
				RTEventTable->PostText( store, 1, CellColor, RTDescription );
				RTEventTable->PostText( store, 2, CellColor, RTEvents[ store ]->accel );
				RTEventTable->PostText( store, 3, CellColor, " " );
			}
			else  if ( TempCurr->ID != 19 ) //OTHERWISE INCREMENT CUE AND ADJUST POINTERS
			{
              	Move = Start;
    			while ( Move->time <= TempCurr->time && Move->next != NULL ) {
    				Move = Move->next;
    			}
    			if ( Move->time > TempCurr->time ) {  //if inserting in the middle
    				eventCue *temp;

    				TempCurr->prev = Move->prev;
    				TempCurr->next = Move;
    				temp = Move->prev;
    				temp->next = TempCurr;
    				Move->prev = TempCurr;
    			}
    			else { //if adding at the end
    				Move->next = TempCurr;
    				TempCurr->prev = Move;
    				TempCurr->next = NULL;
    				End = TempCurr;
    			}
			}
    		nEvents++;
			if ( TempCurr->ID == 19 ) { delete TempCurr; }
		}
	}
	if ( nEvents == 0 ) {
		Error ( "Event list empty...");
	}
    else //Check for absolute calls
	{
     	Move = Start;
		while ( Move != End && Move->absat == 0 )
		{
         	Move = Move->next;
		}
		if ( Move != End && Move->absat == 1 || Move == End && Move->absat == 1 )
		{
			AbsMove = Move;
			//qDebug(AbsMove->text + " AbsEvent %i", AbsMove->time ); //DEBUG	
		}
		else
		{
         	AbsMove = NULL;
			//qDebug( "No abs event found" ); //DEBUG
		}
	}

	Move = Start;

	//Create a copy of a cue
	*StartCopy = *Start;
	MoveCopy = StartCopy;
	if ( Move != End )
	{
       	Move = Start->next;
       	while ( Move != End )
       	{
       		CurrCopy = new eventCue;
       		*CurrCopy = *Move;
       		MoveCopy->next = CurrCopy;
       		CurrCopy->prev = MoveCopy;
       		MoveCopy = CurrCopy;
			//qDebug ( Move->text + "  ||  " + MoveCopy->text );
       		Move = Move->next; 		
       	}
       	if ( Move == End && End != Start )
       	{
       		CurrCopy = new eventCue;
       		*CurrCopy = *Move;
       		MoveCopy->next = CurrCopy;
       		CurrCopy->prev = MoveCopy;
			//qDebug ( Move->text + "  **  " + CurrCopy->text );
			EndCopy = CurrCopy;		
       	}
	}
	//qDebug ( End->text + "  --  " + EndCopy->text );

	Move = Start;	
	MoveCopy = StartCopy;

	Error ( "\nSuccessfully compiled...<br>===============================\n" );
	if ( CompileComments )
	{
		Error("\nWARNING: There were some adjustments and/or comments that were created while compiling the event-script, please see the Console.\n");
	}
	else
	{
     	Error("\n");
	}
    Move = Start;
    compareCompile = Editor->text();
    CompileFlag = FALSE;
    ErrorCompile = FALSE;
    return 0;
}

//DO PURGE OF THE CUE
void RTMix::PurgeCue()
{
    CompileFlag = TRUE;
	Error( "===============================<br>Purging old cue..." );
	//qDebug( "%i", End->ID );
	Move = End;
	MoveCopy = EndCopy;
	while ( Move->prev != NULL ) {
		Curr = Move;
		Move = Move->prev;
       if ( Curr->OSCPtr != NULL )
       {
            delete Curr->OSCPtr;
       }
		delete Curr;
	}
	while ( MoveCopy->prev != NULL ) {
		CurrCopy = MoveCopy;
		MoveCopy = MoveCopy->prev;
       if ( CurrCopy->OSCPtr != NULL )
       {
            delete CurrCopy->OSCPtr;
       }
		delete CurrCopy;
	}
	Start->next = NULL;
	StartCopy->next = NULL;
	End = Start;
	EndCopy = StartCopy;
	ResetRTTable( 1 );
}

void RTMix::AdjustWarnTimer( eventCue *Ptr )
{
	//QString Debug;

	int test, subtract;
	test = int (Ptr->countdown + 0.5);
	if ( test > 59999 ) {
		subtract = test/60000;
		NWarnTenMinute->display ( subtract );
		//Debug.setNum( test, 10 );
		//TextInterface->insert( Debug + " " );
		test = ( test - (subtract * 60000) );
	}
	else {
		NWarnTenMinute->display ( 0 );
	}
	if ( test > 5999 ) {
		subtract = test/6000;
		NWarnMinute->display ( subtract );
		//Debug.setNum( test, 10 );
		//TextInterface->insert( Debug + " " );
		test = ( test - (subtract * 6000) );
	}
	else {
		NWarnMinute->display ( 0 );
	}
	if ( test > 999 ) {
		subtract = test/1000;
		NWarnTenSecond->display ( test/1000 );
		//Debug.setNum( test, 10 );
		//TextInterface->insert( Debug + " " );
		test = ( test - (subtract * 1000) );
	}
	else {
		NWarnTenSecond->display ( 0 );
	}
	if ( test > 99 ) {
		subtract = test/100;
		NWarnSecond->display ( subtract );
		//Debug.setNum( test, 10 );
		//TextInterface->insert( Debug + " " );
		test = ( test - (subtract * 100) );
	}
	else {
		NWarnSecond->display ( 0 );
	}
	if ( test > 9 ) {
		subtract = test/10;
		NWarnTenCentisecond->display ( subtract );
		//Debug.setNum( test, 10 );
		//TextInterface->insert( Debug + " " );
		test = ( test - (subtract * 10 ) );
	}
	else {
		NWarnTenCentisecond->display ( 0 );
	}
	NWarnCentisecond->display ( test );
	//Debug.setNum( test, 10 );
	//TextInterface->insert( Debug + " " );
}

#endif
